@@ -6,15 +6,15 @@
    see the file COPYING for details
    or visit http://www.gnu.org/copyleft/gpl.html
 
-   $Id: iso639.h 3160 2005-12-06 16:17:01Z mosu $
-
    ISO639 language definitions, lookup functions
 
    Written by Moritz Bunkus <moritz@bunkus.org>.
 */
 
-#ifndef __ISO639_H
-#define __ISO639_H
+#ifndef __MTX_COMMON_ISO639_H
+#define __MTX_COMMON_ISO639_H
+
+#include "common/os.h"
 
 typedef struct {
   const char *english_name, *iso639_2_code, *iso639_1_code;
@@ -23,12 +23,11 @@ typedef struct {
 
 extern const iso639_language_t MTX_DLL_API iso639_languages[];
 
-int MTX_DLL_API map_to_iso639_2_code(const char *s,
-                                     bool allow_short_english_names = false);
+int MTX_DLL_API map_to_iso639_2_code(const char *s, bool allow_short_english_names = false);
 bool MTX_DLL_API is_valid_iso639_2_code(const char *iso639_2_code);
 const char *MTX_DLL_API map_iso639_2_to_iso639_1(const char *iso639_2_code);
 void MTX_DLL_API list_iso639_languages();
 bool MTX_DLL_API is_popular_language(const char *lang);
 bool MTX_DLL_API is_popular_language_code(const char *code);
 
-#endif // __ISO639_H
+#endif // __MTX_COMMON_ISO639_H
