@@ -10,15 +10,15 @@
    Written by Moritz Bunkus <moritz@bunkus.org>.
 */
 
-#ifndef __OS_WINDOWS_H
-#define __OS_WINDOWS_H
+#ifndef __MTX_COMMON_OS_WINDOWS_H
+#define __MTX_COMMON_OS_WINDOWS_H
 
-#include "os.h"
-
-#include <string>
+#include "common/os.h"
 
 #ifdef SYS_WINDOWS
 
+#include <string>
+
 int MTX_DLL_API fs_entry_exists(const char *path);
 void MTX_DLL_API create_directory(const char *path);
 int64_t MTX_DLL_API get_current_time_millis();
@@ -31,4 +31,4 @@ std::string MTX_DLL_API get_environment_variable(const std::string &key);
 
 #endif  // SYS_WINDOWS
 
-#endif  // __OS_WINDOWS_H
+#endif  // __MTX_COMMON_OS_WINDOWS_H
