@@ -13,7 +13,7 @@
    \author Steve Lhomme <steve.lhomme@free.fr>.
 */
 
-#include "os.h"
+#include "common/os.h"
 
 #include <ctype.h>
 #include <stdarg.h>
@@ -24,12 +24,11 @@
 #include <matroska/KaxInfo.h>
 #include <matroska/KaxInfoData.h>
 
-#include "common.h"
-#include "commonebml.h"
-#include "error.h"
-#include "segmentinfo.h"
+#include "common/common.h"
+#include "common/ebml.h"
+#include "common/error.h"
+#include "common/segmentinfo.h"
 
-using namespace std;
 using namespace libmatroska;
 
 /** \brief Parse a XML file containing segment info elements.
@@ -46,12 +45,10 @@ using namespace libmatroska;
      \c NULL if an error occured.
 */
 KaxInfo *MTX_DLL_API
-parse_segmentinfo(const string &file_name,
+parse_segmentinfo(const std::string &file_name,
                   bool exception_on_error) {
-  mm_text_io_c *in;
-
   try {
-    in = new mm_text_io_c(new mm_file_io_c(file_name));
+    mm_text_io_c *in = new mm_text_io_c(new mm_file_io_c(file_name));
     return parse_xml_segmentinfo(in, exception_on_error);
   } catch (error_c &e) {
     if (exception_on_error)
@@ -66,7 +63,7 @@ parse_segmentinfo(const string &file_name,
 
    The Matroska specs and \c libmatroska say that several elements are
    mandatory. This function makes sure that they all exist by adding them
-   with their default values if they're missing. It works recursively.
+   with their default values if they're missing.
 
    The parameters are checked for validity.
 
@@ -75,15 +72,14 @@ parse_segmentinfo(const string &file_name,
 */
 void
 fix_mandatory_segmentinfo_elements(EbmlElement *e) {
-  if (e == NULL)
+  if (NULL == e)
     return;
 
-  if (dynamic_cast<EbmlMaster *>(e) != NULL) {
-    EbmlMaster *m;
-    int i;
+  KaxInfo *info = dynamic_cast<KaxInfo *>(e);
+  if (NULL == info)
+    return;
 
-    m = static_cast<EbmlMaster *>(e);
-    for (i = 0; i < m->ListSize(); i++)
-      fix_mandatory_segmentinfo_elements((*m)[i]);
-  }
+  GetChild<KaxTimecodeScale>(info);
+  GetChild<KaxMuxingApp>(info);
+  GetChild<KaxWritingApp>(info);
 }
