@@ -13,16 +13,16 @@
 #ifndef __XTR_MPEG1_2_H
 #define __XTR_MPEG1_2_H
 
-#include "os.h"
+#include "common/os.h"
 
-#include "xtr_base.h"
+#include "extract/xtr_base.h"
 
 class xtr_mpeg1_2_video_c: public xtr_base_c {
 public:
   memory_cptr m_seq_hdr;
 
 public:
-  xtr_mpeg1_2_video_c(const string &codec_id, int64_t tid, track_spec_t &tspec);
+  xtr_mpeg1_2_video_c(const std::string &codec_id, int64_t tid, track_spec_t &tspec);
 
   virtual void create_file(xtr_base_c *master, KaxTrackEntry &track);
   virtual void handle_frame(memory_cptr &frame, KaxBlockAdditions *additions, int64_t timecode, int64_t duration, int64_t bref, int64_t fref,
