@@ -10,19 +10,19 @@
    Written by Moritz Bunkus <moritz@bunkus.org>.
 */
 
-#include "os.h"
+#include "common/os.h"
 
 #include <time.h>
 
-#include "checksums.h"
-#include "commonebml.h"
-#include "tta_common.h"
-
-#include "xtr_tta.h"
+#include "common/checksums.h"
+#include "common/ebml.h"
+#include "common/endian.h"
+#include "common/tta.h"
+#include "extract/xtr_tta.h"
 
 const double xtr_tta_c::ms_tta_frame_time = 1.04489795918367346939l;
 
-xtr_tta_c::xtr_tta_c(const string &codec_id,
+xtr_tta_c::xtr_tta_c(const std::string &codec_id,
                      int64_t tid,
                      track_spec_t &tspec)
   : xtr_base_c(codec_id, tid, tspec)
@@ -61,7 +61,7 @@ xtr_tta_c::handle_frame(memory_cptr &frame,
   m_content_decoder.reverse(frame, CONTENT_ENCODING_SCOPE_BLOCK);
 
   m_frame_sizes.push_back(frame->get_size());
-  m_out->write(frame->get(), frame->get_size());
+  m_out->write(frame);
 
   if (0 < duration)
     m_previous_duration = duration;
