@@ -10,16 +10,16 @@
    Written by Moritz Bunkus <moritz@bunkus.org>.
 */
 
-#include "config.h"
+#include "common/os.h"
 
 #include <ebml/EbmlVersion.h>
 #include <matroska/KaxVersion.h>
 
-#include "common.h"
-#include "mkvinfo.h"
-#include "qtcommon.h"
-
-#include "qt_ui.h"
+#include "common/common.h"
+#include "common/locale.h"
+#include "common/qt.h"
+#include "info/qt_ui.h"
+#include "info/mkvinfo.h"
 
 #include <QMessageBox>
 #include <QMouseEvent>
@@ -276,8 +276,8 @@ ui_show_progress(int percentage,
 int
 ui_run(int argc,
        char **argv) {
-  vector<string> args;
-  string initial_file;
+  std::vector<std::string> args;
+  std::string initial_file;
 
   QApplication app(argc, argv);
   main_window_c main_window;
