@@ -14,15 +14,15 @@
 #ifndef __R_AAC_H
 #define __R_AAC_H
 
-#include "os.h"
+#include "common/os.h"
 
 #include <stdio.h>
 
-#include "aac_common.h"
-#include "common.h"
-#include "error.h"
-#include "mm_io.h"
-#include "pr_generic.h"
+#include "common/aac.h"
+#include "common/common.h"
+#include "common/error.h"
+#include "common/mm_io.h"
+#include "merge/pr_generic.h"
 
 class aac_reader_c: public generic_reader_c {
 private:
@@ -42,10 +42,11 @@ public:
   virtual int get_progress();
   virtual void create_packetizer(int64_t id);
 
-  static int probe_file(mm_io_c *io, int64_t size);
+  static int probe_file(mm_io_c *io, int64_t size, int64_t probe_range, int num_headers);
 
 protected:
   virtual void guess_adts_version();
+  static int find_valid_headers(mm_io_c *io, int64_t probe_range, int num_headers);
 };
 
 #endif // __R_AAC_H
