@@ -12,16 +12,7 @@
    Modified by Steve Lhomme <steve.lhomme@free.fr>.
 */
 
-// {{{ includes
-
-#include "os.h"
-
-#include <ctype.h>
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-
-#include <typeinfo>
+#include "common/os.h"
 
 extern "C" {                    // for BITMAPINFOHEADER
 #include "avilib.h"
@@ -51,37 +42,44 @@ extern "C" {                    // for BITMAPINFOHEADER
 #include <matroska/KaxTrackAudio.h>
 #include <matroska/KaxTrackVideo.h>
 
-#include "chapters.h"
-#include "common.h"
-#include "commonebml.h"
-#include "hacks.h"
-#include "matroska.h"
-#include "output_control.h"
-#include "mm_io.h"
-#include "pr_generic.h"
-#include "r_matroska.h"
-
-#include "p_aac.h"
-#include "p_ac3.h"
-#include "p_avc.h"
-#include "p_dirac.h"
-#include "p_dts.h"
+#include "common/chapters/chapters.h"
+#include "common/common.h"
+#include "common/ebml.h"
+#include "common/endian.h"
+#include "common/hacks.h"
+#include "common/math.h"
+#include "common/matroska.h"
+#include "common/mm_io.h"
+#include "common/strings/formatting.h"
+#include "common/strings/parsing.h"
+#include "common/tags/tags.h"
+#include "input/r_matroska.h"
+#include "merge/output_control.h"
+#include "merge/pr_generic.h"
+#include "output/p_aac.h"
+#include "output/p_ac3.h"
+#include "output/p_avc.h"
+#include "output/p_dirac.h"
+#include "output/p_dts.h"
 #if defined(HAVE_FLAC_FORMAT_H)
-#include "p_flac.h"
+# include "output/p_flac.h"
 #endif
-#include "p_mp3.h"
-#include "p_passthrough.h"
-#include "p_pcm.h"
-#include "p_textsubs.h"
-#include "p_tta.h"
-#include "p_video.h"
-#include "p_vobsub.h"
-#include "p_vobbtn.h"
-#include "p_vorbis.h"
-#include "p_wavpack.h"
-#include "p_kate.h"
+#include "output/p_kate.h"
+#include "output/p_mp3.h"
+#include "output/p_mpeg1_2.h"
+#include "output/p_mpeg4_p2.h"
+#include "output/p_mpeg4_p10.h"
+#include "output/p_passthrough.h"
+#include "output/p_pcm.h"
+#include "output/p_textsubs.h"
+#include "output/p_theora.h"
+#include "output/p_tta.h"
+#include "output/p_video.h"
+#include "output/p_vobbtn.h"
+#include "output/p_vobsub.h"
+#include "output/p_vorbis.h"
+#include "output/p_wavpack.h"
 
-using namespace std;
 using namespace libmatroska;
 
 #define MAP_TRACK_TYPE(c) (  (c) == 'a' ? track_audio   \
@@ -97,12 +95,8 @@ using namespace libmatroska;
 #define in_parent(p) \
   (!p->IsFiniteSize() || (in->getFilePointer() < (p->GetElementPosition() + p->HeadSize() + p->GetSize())))
 
-// }}}
-
 #define is_ebmlvoid(e) (EbmlId(*e) == EbmlVoid::ClassInfos.GlobalId)
 
-// {{{ FUNCTION kax_reader::probe_file()
-
 #define MAGIC_MKV 0x1a45dfa3
 
 /*
@@ -129,10 +123,6 @@ kax_reader_c::probe_file(mm_io_c *io,
   return 1;
 }
 
-// }}}
-
-// {{{ C'TOR
-
 kax_reader_c::kax_reader_c(track_info_c &_ti)
   throw (error_c)
   : generic_reader_c(_ti)
@@ -140,6 +130,7 @@ kax_reader_c::kax_reader_c(track_info_c &_ti)
   , first_timecode(-1)
   , writing_app_ver(-1)
   , m_attachment_id(0)
+  , m_tags(NULL)
 {
   if (!read_headers())
     throw error_c(Y("matroska_reader: Failed to read the headers."));
@@ -147,10 +138,6 @@ kax_reader_c::kax_reader_c(track_info_c &_ti)
     mxinfo_fn(ti.fname, Y("Using the Matroska demultiplexer.\n"));
 }
 
-// }}}
-
-// {{{ D'TOR
-
 kax_reader_c::~kax_reader_c() {
   int i;
 
@@ -161,12 +148,9 @@ kax_reader_c::~kax_reader_c() {
   delete in;
   delete es;
   delete segment;
+  delete m_tags;
 }
 
-// }}}
-
-// {{{ FUNCTIONS packets_available(), new_kax_track(), find_track*
-
 int
 kax_reader_c::packets_available() {
   int i;
@@ -183,14 +167,8 @@ kax_reader_c::packets_available() {
 
 kax_track_t *
 kax_reader_c::new_kax_track() {
-  // Set some default values.
-  kax_track_t *t   = new kax_track_t;
-  t->default_track = true;
-  t->ptzr          = -1;
-
-  tracks.push_back(t);
-
-  return t;
+  tracks.push_back(new kax_track_t);
+  return tracks.back();
 }
 
 kax_track_t *
@@ -217,10 +195,6 @@ kax_reader_c::find_track_by_uid(uint64_t uid,
   return NULL;
 }
 
-// }}}
-
-// {{{ FUNCTION kax_reader_c::verify_tracks()
-
 void
 kax_reader_c::verify_tracks() {
   int tnum, u;
@@ -239,7 +213,7 @@ kax_reader_c::verify_tracks() {
       memory_cptr private_data(new memory_c((unsigned char *)t->private_data, t->private_size, true));
       t->content_decoder.reverse(private_data, CONTENT_ENCODING_SCOPE_CODECPRIVATE);
       private_data->lock();
-      t->private_data = private_data->get();
+      t->private_data = private_data->get_buffer();
       t->private_size = private_data->get_size();
     }
 
@@ -369,13 +343,13 @@ kax_reader_c::verify_tracks() {
 
             try {
               memory_cptr temp(new memory_c((unsigned char *)t->private_data, t->private_size, false));
-              vector<memory_cptr> blocks = unlace_memory_xiph(temp);
+              std::vector<memory_cptr> blocks = unlace_memory_xiph(temp);
               if (blocks.size() != 3)
                 throw false;
 
-              t->headers[0]      = blocks[0]->get();
-              t->headers[1]      = blocks[1]->get();
-              t->headers[2]      = blocks[2]->get();
+              t->headers[0]      = blocks[0]->get_buffer();
+              t->headers[1]      = blocks[1]->get_buffer();
+              t->headers[2]      = blocks[2]->get_buffer();
               t->header_sizes[0] = blocks[0]->get_size();
               t->header_sizes[1] = blocks[1]->get_size();
               t->header_sizes[2] = blocks[2]->get_size();
@@ -473,10 +447,6 @@ kax_reader_c::verify_tracks() {
   }
 }
 
-// }}}
-
-// {{{ FUNCTION kax_reader_c::handle_attachments()
-
 void
 kax_reader_c::handle_attachments(mm_io_c *io,
                                  EbmlElement *l0,
@@ -512,7 +482,7 @@ kax_reader_c::handle_attachments(mm_io_c *io,
       if (EbmlId(*att) == KaxAttached::ClassInfos.GlobalId) {
         UTFstring name        = L"";
         UTFstring description = L"";
-        string mime_type      =  "";
+        std::string mime_type =  "";
         int64_t size          = -1;
         int64_t id            = -1;
         unsigned char *data   = NULL;
@@ -531,7 +501,7 @@ kax_reader_c::handle_attachments(mm_io_c *io,
 
           } else if (EbmlId(*l2) == KaxMimeType::ClassInfos.GlobalId) {
             KaxMimeType &mtype = *static_cast<KaxMimeType *>(l2);
-            mime_type          = string(mtype);
+            mime_type          = std::string(mtype);
 
           } else if (EbmlId(*l2) == KaxFileUID::ClassInfos.GlobalId) {
             KaxFileUID &fuid = *static_cast<KaxFileUID *>(l2);
@@ -645,21 +615,19 @@ kax_reader_c::handle_tags(mm_io_c *io,
         continue;
       }
 
-      bool is_global        = false;
+      bool delete_tag       = true;
+      bool is_global        = true;
       KaxTag *tag           = static_cast<KaxTag *>((*tags)[0]);
       KaxTagTargets *target = FINDFIRST(tag, KaxTagTargets);
 
       if (NULL != target) {
         KaxTagTrackUID *tuid = FINDFIRST(target, KaxTagTrackUID);
 
-        if (NULL == tuid)
-          is_global = true;
-        else {
-          found              = false;
+        if (NULL != tuid) {
+          is_global          = false;
           kax_track_t *track = find_track_by_uid(uint64(*tuid));
 
           if (NULL != track) {
-            found             = true;
             bool contains_tag = false;
 
             for (i = 0; i < tag->ListSize(); i++)
@@ -672,15 +640,19 @@ kax_reader_c::handle_tags(mm_io_c *io,
               if (NULL == track->tags)
                 track->tags = new KaxTags;
               track->tags->PushElement(*tag);
+
+              delete_tag = false;
             }
           }
         }
-      } else
-        is_global = true;
+      }
 
-      if (is_global)
-        add_tags(tag);
-      else if (!found)
+      if (is_global) {
+        if (NULL == m_tags)
+          m_tags = new KaxTags;
+        m_tags->PushElement(*tag);
+
+      } else if (delete_tag)
         delete tag;
 
       tags->Remove(0);
@@ -692,8 +664,6 @@ kax_reader_c::handle_tags(mm_io_c *io,
   io->restore_pos();
 }
 
-// }}}
-
 void
 kax_reader_c::read_headers_info(EbmlElement *&l1,
                                 EbmlElement *&l2,
@@ -733,7 +703,7 @@ kax_reader_c::read_headers_info(EbmlElement *&l1,
   // AVI-Mux GUI 1.16.8 MPEG test build 1, Aug 24 2004  12:42:57
   //
   // The idea is to first replace known application names that contain
-  // spaces with one that doesn't. Then split the whole string up on
+  // spaces with one that doesn't. Then split the whole std::string up on
   // spaces into at most three parts. If the result is at least two parts
   // long then try to parse the version number from the second and
   // store a lower case version of the first as the application's name.
@@ -759,14 +729,14 @@ void
 kax_reader_c::read_headers_info_writing_app(KaxWritingApp *&kwriting_app) {
   int idx;
 
-  string s = UTFstring_to_cstrutf8(UTFstring(*kwriting_app));
+  std::string s = UTFstring_to_cstrutf8(UTFstring(*kwriting_app));
   strip(s);
   mxverb(2, boost::format("matroska_reader: | + writing app: %1%\n") % s);
 
   if (starts_with_case(s, "avi-mux gui"))
     s.replace(0, strlen("avi-mux gui"), "avimuxgui");
 
-  vector<string> parts = split(s.c_str(), " ", 3);
+  std::vector<std::string> parts = split(s.c_str(), " ", 3);
   if (parts.size() < 2) {
     writing_app = "";
     for (idx = 0; idx < s.size(); idx++)
@@ -784,7 +754,7 @@ kax_reader_c::read_headers_info_writing_app(KaxWritingApp *&kwriting_app) {
       if (isdigit(parts[1][idx]) || (parts[1][idx] == '.'))
         s += parts[1][idx];
 
-    vector<string> ver_parts = split(s.c_str(), ".");
+    std::vector<std::string> ver_parts = split(s.c_str(), ".");
     for (idx = ver_parts.size(); idx < 4; idx++)
       ver_parts.push_back("0");
 
@@ -972,8 +942,8 @@ kax_reader_c::read_headers_tracks(EbmlElement *&l1,
 
     KaxCodecID *kcodecid = FINDFIRST(ktentry, KaxCodecID);
     if (NULL != kcodecid) {
-      mxverb(2, boost::format("matroska_reader: |  + Codec ID: %1%\n") % string(*kcodecid));
-      track->codec_id = string(*kcodecid);
+      mxverb(2, boost::format("matroska_reader: |  + Codec ID: %1%\n") % std::string(*kcodecid));
+      track->codec_id = std::string(*kcodecid);
     } else
       mxerror(Y("matroska_reader: The CodecID is missing.\n"));
 
@@ -1027,8 +997,8 @@ kax_reader_c::read_headers_tracks(EbmlElement *&l1,
 
     KaxTrackLanguage *ktlanguage = FINDFIRST(ktentry, KaxTrackLanguage);
     if (NULL != ktlanguage) {
-      mxverb(2, boost::format("matroska_reader: |  + Language: %1%\n") % string(*ktlanguage));
-      track->language = string(*ktlanguage);
+      mxverb(2, boost::format("matroska_reader: |  + Language: %1%\n") % std::string(*ktlanguage));
+      track->language = std::string(*ktlanguage);
     }
 
     KaxTrackName *ktname = FINDFIRST(ktentry, KaxTrackName);
@@ -1047,9 +1017,9 @@ kax_reader_c::read_headers_tracks(EbmlElement *&l1,
 void
 kax_reader_c::read_headers_seek_head(EbmlElement *&l0,
                                      EbmlElement *&l1,
-                                     vector<int64_t> &deferred_tags,
-                                     vector<int64_t> &deferred_chapters,
-                                     vector<int64_t> &deferred_attachments) {
+                                     std::vector<int64_t> &deferred_tags,
+                                     std::vector<int64_t> &deferred_chapters,
+                                     std::vector<int64_t> &deferred_attachments) {
   EbmlElement *el;
 
   KaxSeekHead &seek_head = *static_cast<KaxSeekHead *>(l1);
@@ -1094,12 +1064,10 @@ kax_reader_c::read_headers_seek_head(EbmlElement *&l0,
   }
 }
 
-// {{{ FUNCTION kax_reader_c::read_headers()
-
 int
 kax_reader_c::read_headers() {
   // Elements for different levels
-  vector<int64_t> deferred_tags, deferred_chapters, deferred_attachments;
+  std::vector<int64_t> deferred_tags, deferred_chapters, deferred_attachments;
 
   bool exit_loop = false;
   try {
@@ -1208,9 +1176,11 @@ kax_reader_c::read_headers() {
       for (i = 0; i < deferred_chapters.size(); i++)
         handle_chapters(in, l0, deferred_chapters[i]);
 
-    if (!ti.no_tags)
-      for (i = 0; i < deferred_tags.size(); i++)
-        handle_tags(in, l0, deferred_tags[i]);
+    for (i = 0; i < deferred_tags.size(); i++)
+      handle_tags(in, l0, deferred_tags[i]);
+
+    if (!ti.no_global_tags)
+      process_global_tags();
 
   } catch (...) {
     mxerror(Y("matroska_reader: caught exception\n"));
@@ -1224,9 +1194,17 @@ kax_reader_c::read_headers() {
   return 1;
 }
 
-// }}}
+void
+kax_reader_c::process_global_tags() {
+  if ((NULL == m_tags) || g_identifying)
+    return;
 
-// {{{ FUNCTION kax_reader_c::create_packetizers()
+  int i;
+  for (i = 0; m_tags->ListSize() > i; ++i)
+    add_tags(static_cast<KaxTag *>((*m_tags)[i]));
+
+  m_tags->RemoveAll();
+}
 
 void
 kax_reader_c::init_passthrough_packetizer(kax_track_t *t) {
@@ -1291,8 +1269,10 @@ kax_reader_c::set_packetizer_headers(kax_track_t *t) {
     return;
 
   if (t->default_track)
-    PTZR(t->ptzr)->set_as_default_track(t->type == 'v' ? DEFTRACK_TYPE_VIDEO : t->type == 'a' ? DEFTRACK_TYPE_AUDIO : DEFTRACK_TYPE_SUBS,
-                                        DEFAULT_TRACK_PRIORITY_FROM_SOURCE);
+    PTZR(t->ptzr)->set_as_default_track(t->type == 'v' ? DEFTRACK_TYPE_VIDEO : t->type == 'a' ? DEFTRACK_TYPE_AUDIO : DEFTRACK_TYPE_SUBS, DEFAULT_TRACK_PRIORITY_FROM_SOURCE);
+
+  else if (!boost::logic::indeterminate(t->default_track) && boost::logic::indeterminate(PTZR(t->ptzr)->ti.default_track))
+    PTZR(t->ptzr)->ti.default_track = false;
 
   if (t->forced_track && boost::logic::indeterminate(PTZR(t->ptzr)->ti.forced_track))
     PTZR(t->ptzr)->set_track_forced_flag(true);
@@ -1412,7 +1392,7 @@ kax_reader_c::create_audio_packetizer(kax_track_t *t,
         if (sbr)
           profile        = AAC_PROFILE_SBR;
 
-      } else if (!parse_aac_codec_id(string(t->codec_id), id, profile))
+      } else if (!parse_aac_codec_id(std::string(t->codec_id), id, profile))
         mxerror_tid(ti.fname, t->tnum, boost::format(Y("Malformed codec id '%1%'.\n")) % t->codec_id);
 
     } else {
@@ -1498,7 +1478,7 @@ kax_reader_c::create_subtitle_packetizer(kax_track_t *t,
     t->sub_type = 'v';
 
   } else if (starts_with(t->codec_id, "S_TEXT", 6) || (t->codec_id == "S_SSA") || (t->codec_id == "S_ASS")) {
-    string new_codec_id = ((t->codec_id == "S_SSA") || (t->codec_id == "S_ASS")) ? string("S_TEXT/") + string(&t->codec_id[2]) : t->codec_id;
+    std::string new_codec_id = ((t->codec_id == "S_SSA") || (t->codec_id == "S_ASS")) ? std::string("S_TEXT/") + std::string(&t->codec_id[2]) : t->codec_id;
 
     t->ptzr = add_packetizer(new textsubs_packetizer_c(this, nti, new_codec_id.c_str(), t->private_data, t->private_size, false, true));
     mxinfo_tid(ti.fname, t->tnum, Y("Using the text subtitle output module.\n"));
@@ -1555,7 +1535,7 @@ kax_reader_c::create_packetizer(int64_t tid) {
     nti.language   = t->language;
   if (nti.track_name == "")
     nti.track_name = t->track_name;
-  if (NULL != t->tags)
+  if ((NULL != t->tags) && demuxing_requested('T', t->tnum))
     nti.tags       = dynamic_cast<KaxTags *>(t->tags->Clone());
 
   if (hack_engaged(ENGAGE_FORCE_PASSTHROUGH_PACKETIZER)) {
@@ -1609,7 +1589,7 @@ kax_reader_c::create_mpeg4_p10_es_video_packetizer(kax_track_t *t,
                                                    track_info_c &nti) {
   try {
     read_first_frame(t);
-    if (NULL == t->first_frame_data.get())
+    if (!t->first_frame_data.is_set())
       throw false;
 
     avc_es_parser_c parser;
@@ -1621,7 +1601,7 @@ kax_reader_c::create_mpeg4_p10_es_video_packetizer(kax_track_t *t,
 
     if (sizeof(alBITMAPINFOHEADER) < t->private_size)
       parser.add_bytes((unsigned char *)t->private_data + sizeof(alBITMAPINFOHEADER), t->private_size - sizeof(alBITMAPINFOHEADER));
-    parser.add_bytes(t->first_frame_data->get(), t->first_frame_data->get_size());
+    parser.add_bytes(t->first_frame_data->get_buffer(), t->first_frame_data->get_size());
     parser.flush();
 
     if (!parser.headers_parsed())
@@ -1643,11 +1623,9 @@ kax_reader_c::create_mpeg4_p10_es_video_packetizer(kax_track_t *t,
   }
 }
 
-// }}}
-
 void
 kax_reader_c::read_first_frame(kax_track_t *t) {
-  if ((NULL != t->first_frame_data.get()) || (NULL == saved_l1))
+  if (t->first_frame_data.is_set() || (NULL == saved_l1))
     return;
 
   in->save_pos(saved_l1->GetElementPosition());
@@ -1679,7 +1657,7 @@ kax_reader_c::read_first_frame(kax_track_t *t) {
             if ((NULL == block_track) || (0 == block_simple->NumberFrames()))
               continue;
 
-            if (NULL == block_track->first_frame_data.get()) {
+            if (!block_track->first_frame_data.is_set()) {
               DataBuffer &data_buffer = block_simple->GetBuffer(0);
               block_track->first_frame_data = memory_cptr(new memory_c(data_buffer.Buffer(), data_buffer.Size()));
               block_track->content_decoder.reverse(block_track->first_frame_data, CONTENT_ENCODING_SCOPE_BLOCK);
@@ -1705,7 +1683,7 @@ kax_reader_c::read_first_frame(kax_track_t *t) {
             if ((NULL == block_track) || (0 == block->NumberFrames()))
               continue;
 
-            if (NULL == block_track->first_frame_data.get()) {
+            if (!block_track->first_frame_data.is_set()) {
               DataBuffer &data_buffer       = block->GetBuffer(0);
               block_track->first_frame_data = memory_cptr(new memory_c(data_buffer.Buffer(), data_buffer.Size()));
               block_track->content_decoder.reverse(block_track->first_frame_data, CONTENT_ENCODING_SCOPE_BLOCK);
@@ -1749,8 +1727,6 @@ kax_reader_c::read_first_frame(kax_track_t *t) {
   in->restore_pos();
 }
 
-// {{{ FUNCTION kax_reader_c::read()
-
 file_status_e
 kax_reader_c::read(generic_packetizer_c *requested_ptzr,
                    bool force) {
@@ -1870,10 +1846,10 @@ kax_reader_c::read(generic_packetizer_c *requested_ptzr,
                 block_track->content_decoder.reverse(data, CONTENT_ENCODING_SCOPE_BLOCK);
 
                 if (('s' == block_track->type) && ('t' == block_track->sub_type)) {
-                  if ((2 < data->get_size()) || ((0 < data->get_size()) && (' ' != *data->get()) && (0 != *data->get()) && !iscr(*data->get()))) {
+                  if ((2 < data->get_size()) || ((0 < data->get_size()) && (' ' != *data->get_buffer()) && (0 != *data->get_buffer()) && !iscr(*data->get_buffer()))) {
                     char *lines             = (char *)safemalloc(data->get_size() + 1);
                     lines[data->get_size()] = 0;
-                    memcpy(lines, data->get(), data->get_size());
+                    memcpy(lines, data->get_buffer(), data->get_size());
 
                     PTZR(block_track->ptzr)-> process(new packet_t(new memory_c((unsigned char *)lines, 0, true), last_timecode, block_duration, block_bref, block_fref));
                   }
@@ -1991,10 +1967,10 @@ kax_reader_c::read(generic_packetizer_c *requested_ptzr,
                 block_track->content_decoder.reverse(data, CONTENT_ENCODING_SCOPE_BLOCK);
 
                 if (('s' == block_track->type) && ('t' == block_track->sub_type)) {
-                  if ((2 < data->get_size()) || ((0 < data->get_size()) && (' ' != *data->get()) && (0 != *data->get()) && !iscr(*data->get()))) {
+                  if ((2 < data->get_size()) || ((0 < data->get_size()) && (' ' != *data->get_buffer()) && (0 != *data->get_buffer()) && !iscr(*data->get_buffer()))) {
                     char *lines             = (char *)safemalloc(data->get_size() + 1);
                     lines[data->get_size()] = 0;
-                    memcpy(lines, data->get(), data->get_size());
+                    memcpy(lines, data->get_buffer(), data->get_size());
 
                     packet_t *packet = new packet_t(new memory_c((unsigned char *)lines, 0, true), last_timecode, block_duration, block_bref, block_fref);
                     if (NULL != codec_state)
@@ -2076,10 +2052,6 @@ kax_reader_c::read(generic_packetizer_c *requested_ptzr,
   return FILE_STATUS_DONE;
 }
 
-// }}}
-
-// {{{ FUNCTIONS display_*
-
 int
 kax_reader_c::get_progress() {
   if (0 != segment_duration)
@@ -2098,13 +2070,9 @@ kax_reader_c::set_headers() {
       PTZR(tracks[i]->ptzr)->get_track_entry()->EnableLacing(tracks[i]->lacing_flag);
 }
 
-// }}}
-
-// {{{ FUNCTION kax_reader_c::identify()
-
 void
 kax_reader_c::identify() {
-  vector<string> verbose_info;
+  std::vector<std::string> verbose_info;
 
   if (!title.empty())
     verbose_info.push_back((boost::format("title:%1%") % escape(title)).str());
@@ -2140,10 +2108,10 @@ kax_reader_c::identify() {
     else if (tracks[i]->codec_id == MKV_V_MPEG4_AVC)
       verbose_info.push_back("packetizer:mpeg4_p10_video");
 
-    string info = tracks[i]->codec_id;
+    std::string info = tracks[i]->codec_id;
 
     if (tracks[i]->ms_compat)
-      info += string(", ") +
+      info += std::string(", ") +
         (  tracks[i]->type        == 'v'    ? tracks[i]->v_fourcc
          : tracks[i]->a_formattag == 0x0001 ? "PCM"
          : tracks[i]->a_formattag == 0x0003 ? "PCM"
@@ -2162,9 +2130,17 @@ kax_reader_c::identify() {
 
   for (i = 0; i < g_attachments.size(); i++)
     id_result_attachment(g_attachments[i].ui_id, g_attachments[i].mime_type, g_attachments[i].data->get_size(), g_attachments[i].name, g_attachments[i].description);
-}
 
-// }}}
+  if (NULL != chapters)
+    id_result_chapters(count_chapter_atoms(*chapters));
+
+  if (NULL != m_tags)
+    id_result_tags(ID_RESULT_GLOBAL_TAGS_ID, count_simple_tags(*m_tags));
+
+  for (i = 0; tracks.size() > i; i++)
+    if (tracks[i]->ok && (NULL != tracks[i]->tags))
+      id_result_tags(tracks[i]->tnum, count_simple_tags(*tracks[i]->tags));
+}
 
 void
 kax_reader_c::add_available_track_ids() {
