@@ -13,7 +13,7 @@
    \author Modified by Steve Lhomme <steve.lhomme@free.fr>.
 */
 
-#include "os.h"
+#include "common/os.h"
 
 #include <boost/regex.hpp>
 #include <errno.h>
@@ -40,42 +40,47 @@
 #include <vector>
 
 #include <matroska/KaxChapters.h>
+#include <matroska/KaxInfoData.h>
+#include <matroska/KaxSegment.h>
 #include <matroska/KaxTag.h>
 #include <matroska/KaxTags.h>
 
-#include "chapters.h"
-#include "cluster_helper.h"
-#include "common.h"
-#include "commonebml.h"
-#include "extern_data.h"
-#include "hacks.h"
-#include "iso639.h"
-#include "mkvmerge.h"
-#include "mm_io.h"
-#include "output_control.h"
-#include "segmentinfo.h"
-#include "tagparser.h"
-#include "tag_common.h"
+#include "common/chapters/chapters.h"
+#include "common/command_line.h"
+#include "common/common.h"
+#include "common/ebml.h"
+#include "common/extern_data.h"
+#include "common/hacks.h"
+#include "common/iso639.h"
+#include "common/locale.h"
+#include "common/mm_io.h"
+#include "common/segmentinfo.h"
+#include "common/strings/parsing.h"
+#include "common/tags/tags.h"
+#include "common/tags/parser.h"
+#include "common/unique_numbers.h"
+#include "merge/cluster_helper.h"
+#include "merge/mkvmerge.h"
+#include "merge/output_control.h"
 
 #ifdef SYS_WINDOWS
-# include "os_windows.h"
+# include "common/os_windows.h"
 #endif
 
 using namespace libmatroska;
-using namespace std;
 
 struct ext_file_type_t {
-  string ext, desc;
+  std::string ext, desc;
 
-  ext_file_type_t(const string &p_ext,
-                  const string &p_desc)
+  ext_file_type_t(const std::string &p_ext,
+                  const std::string &p_desc)
     : ext(p_ext)
     , desc(p_desc)
   {
   }
 };
 
-static list<ext_file_type_t> s_input_file_types;
+static std::list<ext_file_type_t> s_input_file_types;
 
 static void
 init_input_file_type_list() {
@@ -184,26 +189,34 @@ set_usage() {
                   "                           firsts Matroska file written.\n");
   usage_text +=   "\n";
   usage_text += Y(" Options for each input file:\n");
-  usage_text += Y("  -a, --atracks <n,m,...>  Copy audio tracks n,m etc. Default: copy all\n"
+  usage_text += Y("  -a, --audio-tracks <n,m,...>\n"
+                  "                           Copy audio tracks n,m etc. Default: copy all\n"
                   "                           audio tracks.\n");
-  usage_text += Y("  -A, --noaudio            Don't copy any audio track from this file.\n");
-  usage_text += Y("  -d, --vtracks <n,m,...>  Copy video tracks n,m etc. Default: copy all\n"
+  usage_text += Y("  -A, --no-audio           Don't copy any audio track from this file.\n");
+  usage_text += Y("  -d, --video-tracks <n,m,...>\n"
+                  "                           Copy video tracks n,m etc. Default: copy all\n"
                   "                           video tracks.\n");
-  usage_text += Y("  -D, --novideo            Don't copy any video track from this file.\n");
-  usage_text += Y("  -s, --stracks <n,m,...>  Copy subtitle tracks n,m etc. Default: copy\n"
+  usage_text += Y("  -D, --no-video           Don't copy any video track from this file.\n");
+  usage_text += Y("  -s, --subtitle-tracks <n,m,...>\n"
+                  "                           Copy subtitle tracks n,m etc. Default: copy\n"
                   "                           all subtitle tracks.\n");
-  usage_text += Y("  -S, --nosubs             Don't copy any text track from this file.\n");
-  usage_text += Y("  -b, --btracks <n,m,...>  Copy buttons tracks n,m etc. Default: copy\n"
+  usage_text += Y("  -S, --no-subtitles       Don't copy any subtitle track from this file.\n");
+  usage_text += Y("  -b, --button-tracks <n,m,...>\n"
+                  "                           Copy buttons tracks n,m etc. Default: copy\n"
                   "                           all buttons tracks.\n");
-  usage_text += Y("  -B, --nobuttons          Don't copy any buttons track from this file.\n");
+  usage_text += Y("  -B, --no-buttons         Don't copy any buttons track from this file.\n");
   usage_text += Y("  -m, --attachments <n[:all|first],m[:all|first],...>\n"
                   "                           Copy the attachments with the IDs n, m etc to\n"
                   "                           all or only the first output file. Default: copy\n"
                   "                           all attachments to all output files.\n");
   usage_text += Y("  -M, --no-attachments     Don't copy attachments from a source file.\n");
-  usage_text += Y("  --no-chapters            Don't keep chapters from a Matroska file.\n");
-  usage_text += Y("  --no-tags                Don't keep tags from a Matroska file.\n");
-  usage_text += Y("  -y, --sync, --delay <TID:d[,o[/p]]>\n"
+  usage_text += Y("  -t, --tags <TID:file>    Read tags for the track from a XML file.\n");
+  usage_text += Y("  --track-tags <n,m,...>   Copy the tags for tracks n,m etc. Default: copy\n"
+                  "                           tags for all tracks.\n");
+  usage_text += Y("  -T, --no-track-tags      Don't copy tags for tracks from the source file.\n");
+  usage_text += Y("  --no-global-tags         Don't keep global tags from the source file.\n");
+  usage_text += Y("  --no-chapters            Don't keep chapters from the source file.\n");
+  usage_text += Y("  -y, --sync <TID:d[,o[/p]]>\n"
                   "                           Synchronize, adjust the track's timecodes with\n"
                   "                           the id TID by 'd' ms.\n"
                   "                           'o/p': Adjust the timecodes by multiplying with\n"
@@ -224,7 +237,6 @@ set_usage() {
                   "                           None at all, only for I frames, for all.\n");
   usage_text += Y("  --language <TID:lang>    Sets the language for the track (ISO639-2\n"
                   "                           code, see --list-languages).\n");
-  usage_text += Y("  -t, --tags <TID:file>    Read tags for the track from a XML file.\n");
   usage_text += Y("  --aac-is-sbr <TID[:0|1]> The track with the ID is HE-AAC/AAC+/SBR-AAC\n"
                   "                           or not. The value ':1' can be omitted.\n");
   usage_text += Y("  --timecodes <TID:file>   Read the timecodes to be used from a file.\n");
@@ -289,8 +301,6 @@ set_usage() {
   version_info = "mkvmerge v" VERSION " ('" VERSIONNAME "')";
 }
 
-static bool s_print_malloc_report = false;
-
 /** \brief Prints information about what has been compiled into mkvmerge
 */
 static void
@@ -352,63 +362,9 @@ create_track_number(generic_reader_c *reader,
   return tnum;
 }
 
-/** \brief Sets the priority mkvmerge runs with
-
-   Depending on the OS different functions are used. On Unix like systems
-   the process is being nice'd if priority is negative ( = less important).
-   Only the super user can increase the priority, but you shouldn't do
-   such work as root anyway.
-   On Windows SetPriorityClass is used.
-*/
-static void
-set_process_priority(int priority) {
-#if defined(SYS_WINDOWS)
-  switch (priority) {
-    case -2:
-      SetPriorityClass(GetCurrentProcess(), IDLE_PRIORITY_CLASS);
-      SetThreadPriority(GetCurrentThread(), THREAD_PRIORITY_IDLE);
-      break;
-    case -1:
-      SetPriorityClass(GetCurrentProcess(), BELOW_NORMAL_PRIORITY_CLASS);
-      SetThreadPriority(GetCurrentThread(), THREAD_PRIORITY_BELOW_NORMAL);
-      break;
-    case 0:
-      SetPriorityClass(GetCurrentProcess(), NORMAL_PRIORITY_CLASS);
-      SetThreadPriority(GetCurrentThread(), THREAD_PRIORITY_NORMAL);
-      break;
-    case 1:
-      SetPriorityClass(GetCurrentProcess(), ABOVE_NORMAL_PRIORITY_CLASS);
-      SetThreadPriority(GetCurrentThread(), THREAD_PRIORITY_ABOVE_NORMAL);
-      break;
-    case 2:
-      SetPriorityClass(GetCurrentProcess(), HIGH_PRIORITY_CLASS);
-      SetThreadPriority(GetCurrentThread(), THREAD_PRIORITY_HIGHEST);
-      break;
-  }
-#else
-  switch (priority) {
-    case -2:
-      nice(19);
-      break;
-    case -1:
-      nice(2);
-      break;
-    case 0:
-      nice(0);
-      break;
-    case 1:
-      nice(-2);
-      break;
-    case 2:
-      nice(-5);
-      break;
-  }
-#endif
-}
-
-static string
-guess_mime_type_and_report(string file_name) {
-  string mime_type = guess_mime_type(file_name, true);
+static std::string
+guess_mime_type_and_report(std::string file_name) {
+  std::string mime_type = guess_mime_type(file_name, true);
   if (mime_type != "") {
     mxinfo(boost::format(Y("Automatic MIME type recognition for '%1%': %2%\n")) % file_name % mime_type);
     return mime_type;
@@ -446,7 +402,7 @@ list_file_types() {
            "  ext   description\n"
            "  ----  --------------------------\n"));
 
-  list<ext_file_type_t>::const_iterator i;
+  std::list<ext_file_type_t>::const_iterator i;
   mxforeach(i, s_input_file_types)
     mxinfo(boost::format("  %|1$-4s|  %2%\n") % i->ext % i->desc);
 }
@@ -458,7 +414,7 @@ list_file_types() {
    and calls its identify function.
 */
 static void
-identify(const string &filename) {
+identify(const std::string &filename) {
   track_info_c ti;
   filelist_t file;
 
@@ -493,14 +449,14 @@ identify(const string &filename) {
    this means 'frames per second'. It returns a number of nanoseconds.
 */
 int64_t
-parse_number_with_unit(const string &s,
-                       const string &subject,
-                       const string &argument,
-                       string display_s = "") {
+parse_number_with_unit(const std::string &s,
+                       const std::string &subject,
+                       const std::string &argument,
+                       std::string display_s = "") {
   boost::regex re1("(-?\\d+\\.?\\d*)(s|ms|us|ns|fps)?", boost::regex::perl | boost::regbase::icase);
   boost::regex re2("(-?\\d+)/(-?\\d+)(s|ms|us|ns|fps)?", boost::regex::perl | boost::regbase::icase);
 
-  string unit, s_n, s_d;
+  std::string unit, s_n, s_d;
   int64_t n, d;
   double d_value = 0.0;
   bool is_fraction = false;
@@ -560,8 +516,8 @@ parse_number_with_unit(const string &s,
    as boolean false.
 */
 bool
-parse_bool(const string &orig) {
-  string s(downcase(orig));
+parse_bool(const std::string &orig) {
+  std::string s(downcase(orig));
 
   if ((s == "yes") || (s == "true") || (s == "1"))
     return true;
@@ -575,7 +531,7 @@ parse_bool(const string &orig) {
    Also tests the tags for missing mandatory elements.
 */
 void
-parse_and_add_tags(const string &file_name) {
+parse_and_add_tags(const std::string &file_name) {
   KaxTags *tags = new KaxTags;
 
   parse_xml_tags(file_name, tags);
@@ -600,12 +556,12 @@ parse_and_add_tags(const string &file_name) {
    The argument is a comma separated list of track IDs.
 */
 static void
-parse_arg_tracks(string s,
-                 vector<int64_t> &tracks,
-                 const string &opt) {
+parse_arg_tracks(std::string s,
+                 std::vector<int64_t> &tracks,
+                 const std::string &opt) {
   tracks.clear();
 
-  vector<string> elements = split(s, ",");
+  std::vector<std::string> elements = split(s, ",");
   strip(elements);
 
   int i;
@@ -626,14 +582,14 @@ parse_arg_tracks(string s,
    to 1 if not given.
 */
 static void
-parse_arg_sync(string s,
-               const string &opt,
+parse_arg_sync(std::string s,
+               const std::string &opt,
                track_info_c &ti) {
   timecode_sync_t tcsync;
 
   // Extract the track number.
-  string orig          = s;
-  vector<string> parts = split(s, ":", 2);
+  std::string orig          = s;
+  std::vector<std::string> parts = split(s, ":", 2);
   if (parts.size() != 2)
     mxerror(boost::format(Y("Invalid sync option. No track ID specified in '%1% %2%'.\n")) % opt % s);
 
@@ -645,10 +601,15 @@ parse_arg_sync(string s,
   if (s.size() == 0)
     mxerror(boost::format(Y("Invalid sync option specified in '%1% %2%'.\n")) % opt % orig);
 
+  if (parts[1] == "reset") {
+    ti.reset_timecodes_specs[id] = true;
+    return;
+  }
+
   // Now parse the actual sync values.
   int idx = s.find(',');
   if (idx >= 0) {
-    string linear = s.substr(idx + 1);
+    std::string linear = s.substr(idx + 1);
     s.erase(idx);
     idx = linear.find('/');
     if (idx < 0) {
@@ -656,7 +617,7 @@ parse_arg_sync(string s,
       tcsync.denominator = 1.0;
 
     } else {
-      string div = linear.substr(idx + 1);
+      std::string div = linear.substr(idx + 1);
       linear.erase(idx);
       double d1  = strtod(linear.c_str(), NULL);
       double d2  = strtod(div.c_str(), NULL);
@@ -681,16 +642,16 @@ parse_arg_sync(string s,
    The argument must have the form \c TID:w/h or \c TID:float, e.g. \c 0:16/9
 */
 static void
-parse_arg_aspect_ratio(const string &s,
-                       const string &opt,
+parse_arg_aspect_ratio(const std::string &s,
+                       const std::string &opt,
                        bool is_factor,
                        track_info_c &ti) {
   display_properties_t dprop;
 
-  string msg           = is_factor ? Y("Aspect ratio factor") : Y("Aspect ratio");
+  std::string msg      = is_factor ? Y("Aspect ratio factor") : Y("Aspect ratio");
 
   dprop.ar_factor      = is_factor;
-  vector<string> parts = split(s, ":", 2);
+  std::vector<std::string> parts = split(s, ":", 2);
   if (parts.size() != 2)
     mxerror(boost::format(Y("%1%: missing track ID in '%2% %3%'.\n")) % msg % opt % s);
 
@@ -710,7 +671,7 @@ parse_arg_aspect_ratio(const string &s,
     return;
   }
 
-  string div = parts[1].substr(idx + 1);
+  std::string div = parts[1].substr(idx + 1);
   parts[1].erase(idx);
   if (parts[1].empty())
     mxerror(boost::format(Y("%1%: missing dividend in '%2% %3%'.\n")) % msg % opt %s);
@@ -732,16 +693,16 @@ parse_arg_aspect_ratio(const string &s,
    The argument must have the form \c TID:wxh, e.g. \c 0:640x480.
 */
 static void
-parse_arg_display_dimensions(const string s,
+parse_arg_display_dimensions(const std::string s,
                              track_info_c &ti) {
   display_properties_t dprop;
 
-  vector<string> parts = split(s, ":", 2);
+  std::vector<std::string> parts = split(s, ":", 2);
   strip(parts);
   if (parts.size() != 2)
     mxerror(boost::format(Y("Display dimensions: not given in the form <TID>:<width>x<height>, e.g. 1:640x480 (argument was '%1%').\n")) % s);
 
-  vector<string> dims = split(parts[1], "x", 2);
+  std::vector<std::string> dims = split(parts[1], "x", 2);
   int64_t id = 0;
   int w, h;
   if ((dims.size() != 2) || !parse_int(parts[0], id) || !parse_int(dims[0], w) || !parse_int(dims[1], h) || (0 >= w) || (0 >= h))
@@ -760,13 +721,13 @@ parse_arg_display_dimensions(const string s,
    \c 0:10,5,10,5
 */
 static void
-parse_arg_cropping(const string &s,
+parse_arg_cropping(const std::string &s,
                    track_info_c &ti) {
   pixel_crop_t crop;
 
-  string err_msg   = Y("Cropping parameters: not given in the form <TID>:<left>,<top>,<right>,<bottom> e.g. 0:10,5,10,5 (argument was '%1%').\n");
+  std::string err_msg        = Y("Cropping parameters: not given in the form <TID>:<left>,<top>,<right>,<bottom> e.g. 0:10,5,10,5 (argument was '%1%').\n");
 
-  vector<string> v = split(s, ":");
+  std::vector<std::string> v = split(s, ":");
   if (v.size() != 2)
     mxerror(boost::format(err_msg) % s);
 
@@ -790,15 +751,15 @@ parse_arg_cropping(const string &s,
    one of the keywords \c 'none', \c 'left', \c 'right' or \c 'both'.
 */
 static void
-parse_arg_stereo_mode(const string &s,
+parse_arg_stereo_mode(const std::string &s,
                       track_info_c &ti) {
   static const char * const keywords[] = {
     "none", "right", "left", "both", NULL
   };
-  string errmsg = Y("Stereo mode parameter: not given in the form <TID>:<n|keyword> where n is a number between 0 and 3 "
-                    "or one of the keywords 'none', 'right', 'left', 'both' (argument was '%1%').\n");
+  std::string errmsg = Y("Stereo mode parameter: not given in the form <TID>:<n|keyword> where n is a number between 0 and 3 "
+                         "or one of the keywords 'none', 'right', 'left', 'both' (argument was '%1%').\n");
 
-  vector<string> v = split(s, ":");
+  std::vector<std::string> v = split(s, ":");
   if (v.size() != 2)
     mxerror(boost::format(errmsg) % s);
 
@@ -825,8 +786,8 @@ parse_arg_stereo_mode(const string &s,
   a duration after which a new file should be started.
 */
 static void
-parse_arg_split_duration(const string &arg) {
-  string s = arg;
+parse_arg_split_duration(const std::string &arg) {
+  std::string s = arg;
 
   if (starts_with_case(s, "duration:"))
     s.erase(0, strlen("duration:"));
@@ -844,14 +805,14 @@ parse_arg_split_duration(const string &arg) {
   a timecodes after which a new file should be started.
 */
 static void
-parse_arg_split_timecodes(const string &arg) {
-  string s = arg;
+parse_arg_split_timecodes(const std::string &arg) {
+  std::string s = arg;
 
   if (starts_with_case(s, "timecodes:"))
     s.erase(0, 10);
 
-  vector<string> timecodes = split(s, ",");
-  vector<string>::const_iterator timecode;
+  std::vector<std::string> timecodes = split(s, ",");
+  std::vector<std::string>::const_iterator timecode;
   mxforeach(timecode, timecodes) {
     int64_t split_after;
     if (!parse_timecode(*timecode, split_after))
@@ -866,9 +827,9 @@ parse_arg_split_timecodes(const string &arg) {
   a size after which a new file should be started.
 */
 static void
-parse_arg_split_size(const string &arg) {
-  string s       = arg;
-  string err_msg = Y("Invalid split size in '--split %1%'.\n");
+parse_arg_split_size(const std::string &arg) {
+  std::string s       = arg;
+  std::string err_msg = Y("Invalid split size in '--split %1%'.\n");
 
   if (starts_with_case(s, "size:"))
     s.erase(0, strlen("size:"));
@@ -913,13 +874,13 @@ parse_arg_split_size(const string &arg) {
    This format is parsed by ::parse_split_duration
 */
 static void
-parse_arg_split(const string &arg) {
-  string err_msg = Y("Invalid format for '--split' in '--split %1%'.\n");
+parse_arg_split(const std::string &arg) {
+  std::string err_msg = Y("Invalid format for '--split' in '--split %1%'.\n");
 
   if (arg.size() < 2)
     mxerror(boost::format(err_msg) % arg);
 
-  string s = arg;
+  std::string s = arg;
 
   // HH:MM:SS
   if (starts_with_case(s, "duration:"))
@@ -959,10 +920,10 @@ parse_arg_split(const string &arg) {
    is equivalent to \c TID:1.
 */
 static void
-parse_arg_default_track(const string &s,
+parse_arg_default_track(const std::string &s,
                         track_info_c &ti) {
   bool is_default      = true;
-  vector<string> parts = split(s, ":", 2);
+  std::vector<std::string> parts = split(s, ":", 2);
   int64_t id           = 0;
 
   strip(parts);
@@ -985,11 +946,11 @@ parse_arg_default_track(const string &s,
    is equivalent to \c TID:1.
 */
 static void
-parse_arg_forced_track(const string &s,
+parse_arg_forced_track(const std::string &s,
                         track_info_c &ti) {
-  bool is_forced       = true;
-  vector<string> parts = split(s, ":", 2);
-  int64_t id           = 0;
+  bool is_forced                 = true;
+  std::vector<std::string> parts = split(s, ":", 2);
+  int64_t id                     = 0;
 
   strip(parts);
   if (!parse_int(parts[0], id))
@@ -1010,11 +971,11 @@ parse_arg_forced_track(const string &s,
    The argument must have the form \c TID:cuestyle, e.g. \c 0:none.
 */
 static void
-parse_arg_cues(const string &s,
+parse_arg_cues(const std::string &s,
                track_info_c &ti) {
 
   // Extract the track number.
-  vector<string> parts = split(s, ":", 2);
+  std::vector<std::string> parts = split(s, ":", 2);
   strip(parts);
   if (parts.size() != 2)
     mxerror(boost::format(Y("Invalid cues option. No track ID specified in '--cues %1%'.\n")) % s);
@@ -1041,10 +1002,10 @@ parse_arg_cues(const string &s,
    The argument must have the form \c TID:compression, e.g. \c 0:bz2.
 */
 static void
-parse_arg_compression(const string &s,
+parse_arg_compression(const std::string &s,
                   track_info_c &ti) {
   // Extract the track number.
-  vector<string> parts = split(s, ":", 2);
+  std::vector<std::string> parts = split(s, ":", 2);
   strip(parts);
   if (parts.size() != 2)
     mxerror(boost::format(Y("Invalid compression option. No track ID specified in '--compression %1%'.\n")) % s);
@@ -1082,14 +1043,14 @@ parse_arg_compression(const string &s,
    the form \c TID:value, e.g. \c 0:XVID.
 */
 static void
-parse_arg_language(const string &s,
-                   map<int64_t, string> &storage,
-                   const string &opt,
+parse_arg_language(const std::string &s,
+                   std::map<int64_t, std::string> &storage,
+                   const std::string &opt,
                    const char *topic,
                    bool check,
                    bool empty_ok = false) {
   // Extract the track number.
-  vector<string>parts = split(s, ":", 2);
+  std::vector<std::string>parts = split(s, ":", 2);
   strip(parts);
   if (parts.empty())
     mxerror(boost::format(Y("No track ID specified in '--%1% %2%'.\n")) % opt % s);
@@ -1123,10 +1084,10 @@ parse_arg_language(const string &s,
    The argument must have the form \c TID:charset, e.g. \c 0:ISO8859-15.
 */
 static void
-parse_arg_sub_charset(const string &s,
+parse_arg_sub_charset(const std::string &s,
                       track_info_c &ti) {
   // Extract the track number.
-  vector<string> parts = split(s, ":", 2);
+  std::vector<std::string> parts = split(s, ":", 2);
   strip(parts);
   if (parts.size() != 2)
     mxerror(boost::format(Y("Invalid sub charset option. No track ID specified in '--sub-charset %1%'.\n")) % s);
@@ -1146,11 +1107,11 @@ parse_arg_sub_charset(const string &s,
    The argument must have the form \c TID:filename, e.g. \c 0:tags.xml.
 */
 static void
-parse_arg_tags(const string &s,
-               const string &opt,
+parse_arg_tags(const std::string &s,
+               const std::string &opt,
                track_info_c &ti) {
   // Extract the track number.
-  vector<string> parts = split(s, ":", 2);
+  std::vector<std::string> parts = split(s, ":", 2);
   strip(parts);
   if (parts.size() != 2)
     mxerror(boost::format(Y("Invalid tags option. No track ID specified in '%1% %2%'.\n")) % opt % s);
@@ -1170,11 +1131,11 @@ parse_arg_tags(const string &s,
    The argument must have the form \c TID:fourcc, e.g. \c 0:XVID.
 */
 static void
-parse_arg_fourcc(const string &s,
-                 const string &opt,
+parse_arg_fourcc(const std::string &s,
+                 const std::string &opt,
                  track_info_c &ti) {
-  string orig          = s;
-  vector<string> parts = split(s, ":", 2);
+  std::string orig               = s;
+  std::vector<std::string> parts = split(s, ":", 2);
   strip(parts);
 
   if (parts.size() != 2)
@@ -1195,15 +1156,15 @@ parse_arg_fourcc(const string &s,
    The argument must be a comma separated list of track IDs.
 */
 static void
-parse_arg_track_order(const string &s) {
+parse_arg_track_order(const std::string &s) {
   track_order_t to;
 
-  vector<string> parts = split(s, ",");
+  std::vector<std::string> parts = split(s, ",");
   strip(parts);
 
   int i;
   for (i = 0; i < parts.size(); i++) {
-    vector<string> pair = split(parts[i].c_str(), ":");
+    std::vector<std::string> pair = split(parts[i].c_str(), ":");
 
     if (pair.size() != 2)
       mxerror(boost::format(Y("'%1%' is not a valid pair of file ID and track ID in '--track-order %2%'.\n")) % parts[i] % s);
@@ -1236,13 +1197,13 @@ parse_arg_track_order(const string &s) {
    appended to the one specified by the "source" file and track ID.
 */
 static void
-parse_arg_append_to(const string &s,
+parse_arg_append_to(const std::string &s,
                     track_info_c &ti) {
   g_append_mapping.clear();
-  vector<string> entries = split(s, ",");
+  std::vector<std::string> entries = split(s, ",");
   strip(entries);
 
-  vector<string>::const_iterator entry;
+  std::vector<std::string>::const_iterator entry;
   mxforeach(entry, entries) {
     append_spec_t mapping;
 
@@ -1250,7 +1211,7 @@ parse_arg_append_to(const string &s,
         || (0 > mapping.src_file_id)
         || (0 > mapping.src_track_id)
         || (0 > mapping.dst_file_id)
-        || (0> mapping.dst_track_id))
+        || (0 > mapping.dst_track_id))
       mxerror(boost::format(Y("'%1%' is not a valid mapping of file and track IDs in '--append-to %2%'.\n")) % (*entry) % s);
 
     g_append_mapping.push_back(mapping);
@@ -1258,7 +1219,7 @@ parse_arg_append_to(const string &s,
 }
 
 static void
-parse_arg_append_mode(const string &s,
+parse_arg_append_mode(const std::string &s,
                       track_info_c &ti) {
   if ((s == "track") || (s == "track-based"))
     g_append_mode = APPEND_MODE_TRACK_BASED;
@@ -1277,9 +1238,9 @@ parse_arg_append_mode(const string &s,
    'us', 'ns' or 'fps' (see \c parse_number_with_unit).
 */
 static void
-parse_arg_default_duration(const string &s,
+parse_arg_default_duration(const std::string &s,
                            track_info_c &ti) {
-  vector<string> parts = split(s, ":");
+  std::vector<std::string> parts = split(s, ":");
   if (parts.size() != 2)
     mxerror(boost::format(Y("'%1%' is not a valid tuple of track ID and default duration in '--default-duration %1%'.\n")) % s);
 
@@ -1296,11 +1257,11 @@ parse_arg_default_duration(const string &s,
    length, an integer between 2 and 4 inclusively.
 */
 static void
-parse_arg_nalu_size_length(const string &s,
+parse_arg_nalu_size_length(const std::string &s,
                            track_info_c &ti) {
   static bool s_nalu_size_length_3_warning_printed = false;
 
-  vector<string> parts = split(s, ":");
+  std::vector<std::string> parts = split(s, ":");
   if (parts.size() != 2)
     mxerror(boost::format(Y("'%1%' is not a valid tuple of track ID and NALU size length in '--nalu-size-length %1%'.\n")) % s);
 
@@ -1326,9 +1287,9 @@ parse_arg_nalu_size_length(const string &s,
    of BlockAdditional IDs.
 */
 static void
-parse_arg_max_blockadd_id(const string &s,
+parse_arg_max_blockadd_id(const std::string &s,
                           track_info_c &ti) {
-  vector<string> parts = split(s, ":");
+  std::vector<std::string> parts = split(s, ":");
   if (parts.size() != 2)
     mxerror(boost::format(Y("'%1%' is not a valid pair of track ID and block additional in '--blockadd %1%'.\n")) % s);
 
@@ -1350,9 +1311,9 @@ parse_arg_max_blockadd_id(const string &s,
    a track ID is given then "number" is assumed to be "1".
 */
 static void
-parse_arg_aac_is_sbr(const string &s,
+parse_arg_aac_is_sbr(const std::string &s,
                      track_info_c &ti) {
-  vector<string> parts = split(s, ":", 2);
+  std::vector<std::string> parts = split(s, ":", 2);
 
   int64_t id = 0;
   if (!parse_int(parts[0], id) || (id < 0))
@@ -1365,7 +1326,7 @@ parse_arg_aac_is_sbr(const string &s,
 }
 
 static void
-parse_arg_priority(const string &arg) {
+parse_arg_priority(const std::string &arg) {
   static const char *s_process_priorities[5] = {"lowest", "lower", "normal", "higher", "highest"};
 
   int i;
@@ -1379,9 +1340,9 @@ parse_arg_priority(const string &arg) {
 }
 
 static void
-parse_arg_previous_segment_uid(const string &param,
-                               const string &arg) {
-  if (NULL != g_seguid_link_previous.get())
+parse_arg_previous_segment_uid(const std::string &param,
+                               const std::string &arg) {
+  if (g_seguid_link_previous.is_set())
     mxerror(boost::format(Y("The previous UID was already given in '%1% %2%'.\n")) % param % arg);
 
   try {
@@ -1392,9 +1353,9 @@ parse_arg_previous_segment_uid(const string &param,
 }
 
 static void
-parse_arg_next_segment_uid(const string &param,
-                           const string &arg) {
-  if (NULL != g_seguid_link_next.get())
+parse_arg_next_segment_uid(const std::string &param,
+                           const std::string &arg) {
+  if (g_seguid_link_next.is_set())
     mxerror(boost::format(Y("The next UID was already given in '%1% %2%'.\n")) % param % arg);
 
   try {
@@ -1405,19 +1366,20 @@ parse_arg_next_segment_uid(const string &param,
 }
 
 static void
-parse_arg_cluster_length(string arg) {
+parse_arg_cluster_length(std::string arg) {
   int idx = arg.find("ms");
   if (0 <= idx) {
     arg.erase(idx);
-    if (!parse_int(arg, g_max_ns_per_cluster) || (100 > g_max_ns_per_cluster) || (32000 < g_max_ns_per_cluster))
+    int64_t max_ms_per_cluster;
+    if (!parse_int(arg, max_ms_per_cluster) || (100 > max_ms_per_cluster) || (32000 < max_ms_per_cluster))
       mxerror(boost::format(Y("Cluster length '%1%' out of range (100..32000).\n")) % arg);
 
-    g_max_ns_per_cluster     *= 1000000;
-    g_max_blocks_per_cluster  = 65535;
+    g_max_ns_per_cluster     = max_ms_per_cluster * 1000000;
+    g_max_blocks_per_cluster = 65535;
 
   } else {
-    if (!parse_int(arg, g_max_blocks_per_cluster) || (0 > g_max_blocks_per_cluster) || (60000 < g_max_blocks_per_cluster))
-      mxerror(boost::format(Y("Cluster length '%1%' out of range (0..60000).\n")) % arg);
+    if (!parse_int(arg, g_max_blocks_per_cluster) || (0 > g_max_blocks_per_cluster) || (65535 < g_max_blocks_per_cluster))
+      mxerror(boost::format(Y("Cluster length '%1%' out of range (0..65535).\n")) % arg);
 
     g_max_ns_per_cluster = 32000000000ull;
   }
@@ -1425,7 +1387,7 @@ parse_arg_cluster_length(string arg) {
 
 static void
 parse_arg_attach_file(attachment_t &attachment,
-                      const string &arg,
+                      const std::string &arg,
                       bool attach_once) {
   try {
     mm_file_io_c test(arg);
@@ -1446,7 +1408,7 @@ parse_arg_attach_file(attachment_t &attachment,
       mxerror(boost::format(Y("The size of attachment '%1%' is 0.\n")) % attachment.name);
 
     attachment.data = memory_c::alloc(io->get_size());
-    io->read(attachment.data->get(), attachment.data->get_size());
+    io->read(attachment.data->get_buffer(), attachment.data->get_size());
 
   } catch (...) {
     mxerror(boost::format(Y("The attachment '%1%' could not be read.\n")) % attachment.name);
@@ -1457,7 +1419,8 @@ parse_arg_attach_file(attachment_t &attachment,
 }
 
 static void
-parse_arg_chapter_language(const string &arg) {
+parse_arg_chapter_language(const std::string &arg,
+                           track_info_c &ti) {
   if (g_chapter_language != "")
     mxerror(boost::format(Y("'--chapter-language' may only be given once in '--chapter-language %1%'.\n")) % arg);
 
@@ -1469,11 +1432,12 @@ parse_arg_chapter_language(const string &arg) {
     mxerror(boost::format(Y("'%1%' is neither a valid ISO639-2 nor a valid ISO639-1 code in '--chapter-language %1%'. "
                             "See 'mkvmerge --list-languages' for a list of all languages and their respective ISO639-2 codes.\n")) % arg);
 
-  g_chapter_language = iso639_languages[i].iso639_2_code;
+  g_chapter_language  = iso639_languages[i].iso639_2_code;
+  ti.chapter_language = iso639_languages[i].iso639_2_code;
 }
 
 static void
-parse_arg_chapter_charset(const string &arg,
+parse_arg_chapter_charset(const std::string &arg,
                           track_info_c &ti) {
   if (g_chapter_charset != "")
     mxerror(boost::format(Y("'--chapter-charset' may only be given once in '--chapter-charset %1%'.\n")) % arg);
@@ -1486,8 +1450,8 @@ parse_arg_chapter_charset(const string &arg,
 }
 
 static void
-parse_arg_chapters(const string &param,
-                   const string &arg) {
+parse_arg_chapters(const std::string &param,
+                   const std::string &arg) {
   if (g_chapter_file_name != "")
     mxerror(boost::format(Y("Only one chapter file allowed in '%1% %2%'.\n")) % param % arg);
 
@@ -1498,8 +1462,8 @@ parse_arg_chapters(const string &param,
 }
 
 static void
-parse_arg_segmentinfo(const string &param,
-                      const string &arg) {
+parse_arg_segmentinfo(const std::string &param,
+                      const std::string &arg) {
   if (g_segmentinfo_file_name != "")
     mxerror(boost::format(Y("Only one segment info file allowed in '%1% %2%'.\n")) % param % arg);
 
@@ -1510,7 +1474,7 @@ parse_arg_segmentinfo(const string &param,
 }
 
 static void
-parse_arg_timecode_scale(const string &arg) {
+parse_arg_timecode_scale(const std::string &arg) {
   if (TIMECODE_SCALE_MODE_NORMAL != g_timecode_scale_mode)
     mxerror(Y("'--timecode-scale' was used more than once.\n"));
 
@@ -1530,7 +1494,7 @@ parse_arg_timecode_scale(const string &arg) {
 }
 
 static void
-parse_arg_default_language(const string &arg) {
+parse_arg_default_language(const std::string &arg) {
   int i = map_to_iso639_2_code(arg.c_str());
   if (-1 == i)
     mxerror(boost::format(Y("'%1%' is neither a valid ISO639-2 nor a valid ISO639-1 code in '--default-language %1%'. "
@@ -1540,14 +1504,14 @@ parse_arg_default_language(const string &arg) {
 }
 
 static void
-parse_arg_attachments(const string &param,
-                      const string &arg,
+parse_arg_attachments(const std::string &param,
+                      const std::string &arg,
                       track_info_c &ti) {
-  vector<string> elements = split(arg, ",");
+  std::vector<std::string> elements = split(arg, ",");
 
   int i;
   for (i = 0; elements.size() > i; ++i) {
-    vector<string> pair = split(elements[i], ":");
+    std::vector<std::string> pair = split(elements[i], ":");
 
     if (1 == pair.size())
       pair.push_back("all");
@@ -1584,7 +1548,7 @@ parse_arg_attachments(const string &param,
    everything else.
 */
 static void
-parse_args(vector<string> args) {
+parse_args(std::vector<std::string> args) {
   set_usage();
   while (handle_common_cli_args(args, ""))
     set_usage();
@@ -1614,10 +1578,10 @@ parse_args(vector<string> args) {
   }
 
   // First parse options that either just print some infos and then exit.
-  vector<string>::const_iterator sit;
+  std::vector<std::string>::const_iterator sit;
   mxforeach(sit, args) {
-    const string &this_arg = *sit;
-    string next_arg        = ((sit + 1) == args.end()) ? "" : *(sit + 1);
+    const std::string &this_arg = *sit;
+    std::string next_arg        = ((sit + 1) == args.end()) ? "" : *(sit + 1);
 
     if ((this_arg == "-l") || (this_arg == "--list-types")) {
       list_file_types();
@@ -1642,8 +1606,8 @@ parse_args(vector<string> args) {
 
   // Now parse options that are needed right at the beginning.
   mxforeach(sit, args) {
-    const string &this_arg = *sit;
-    string next_arg        = ((sit + 1) == args.end()) ? "" : *(sit + 1);
+    const std::string &this_arg = *sit;
+    std::string next_arg        = ((sit + 1) == args.end()) ? "" : *(sit + 1);
 
     bool no_next_arg       = (sit + 1) == args.end();
 
@@ -1675,8 +1639,8 @@ parse_args(vector<string> args) {
   attachment_t attachment;
 
   mxforeach(sit, args) {
-    const string &this_arg = *sit;
-    string next_arg        = ((sit + 1) == args.end()) ? "" : *(sit + 1);
+    const std::string &this_arg = *sit;
+    std::string next_arg        = ((sit + 1) == args.end()) ? "" : *(sit + 1);
 
     bool no_next_arg       = (sit + 1) == args.end();
 
@@ -1814,7 +1778,7 @@ parse_args(vector<string> args) {
       if (no_next_arg)
         mxerror(Y("'--chapter-language' lacks the language.\n"));
 
-      parse_arg_chapter_language(next_arg);
+      parse_arg_chapter_language(next_arg, *ti);
       sit++;
 
     } else if (this_arg == "--chapter-charset") {
@@ -1831,7 +1795,7 @@ parse_args(vector<string> args) {
       if (g_chapter_file_name != "")
         mxerror(Y("'--cue-chapter-name-format' must be given before '--chapters'.\n"));
 
-      cue_to_chapter_name_format = next_arg;
+      g_cue_to_chapter_name_format = next_arg;
       sit++;
 
     } else if (this_arg == "--chapters") {
@@ -1865,8 +1829,8 @@ parse_args(vector<string> args) {
       parse_arg_attachments(this_arg, next_arg, *ti);
       sit++;
 
-    } else if (this_arg == "--no-tags") {
-      ti->no_tags = true;
+    } else if (this_arg == "--no-global-tags") {
+      ti->no_global_tags = true;
 
     } else if (this_arg == "--meta-seek-size") {
       mxwarn(Y("The option '--meta-seek-size' is no longer supported. Please read mkvmerge's documentation, especially the section about the MATROSKA FILE LAYOUT.\n"));
@@ -1881,46 +1845,56 @@ parse_args(vector<string> args) {
     }
 
     // Options that apply to the next input file only.
-    else if ((this_arg == "-A") || (this_arg == "--noaudio"))
+    else if ((this_arg == "-A") || (this_arg == "--noaudio") || (this_arg == "--no-audio"))
       ti->no_audio = true;
 
-    else if ((this_arg == "-D") || (this_arg == "--novideo"))
+    else if ((this_arg == "-D") || (this_arg == "--novideo") || (this_arg == "--no-video"))
       ti->no_video = true;
 
-    else if ((this_arg == "-S") || (this_arg == "--nosubs"))
+    else if ((this_arg == "-S") || (this_arg == "--nosubs") || (this_arg == "--no-subs") || (this_arg == "--no-subtitles"))
       ti->no_subs = true;
 
-    else if ((this_arg == "-B") || (this_arg == "--nobuttons"))
+    else if ((this_arg == "-B") || (this_arg == "--nobuttons") || (this_arg == "--no-buttons"))
       ti->no_buttons = true;
 
-    else if ((this_arg == "-a") || (this_arg == "--atracks")) {
+    else if ((this_arg == "-T") || (this_arg == "--no-track-tags"))
+      ti->no_track_tags = true;
+
+    else if ((this_arg == "-a") || (this_arg == "--atracks") || (this_arg == "--audio-tracks")) {
       if (no_next_arg)
         mxerror(boost::format(Y("'%1%' lacks the track number(s).\n")) % this_arg);
 
       parse_arg_tracks(next_arg, ti->atracks, this_arg);
       sit++;
 
-    } else if ((this_arg == "-d") || (this_arg == "--vtracks")) {
+    } else if ((this_arg == "-d") || (this_arg == "--vtracks") || (this_arg == "--video-tracks")) {
       if (no_next_arg)
         mxerror(boost::format(Y("'%1%' lacks the track number(s).\n")) % this_arg);
 
       parse_arg_tracks(next_arg, ti->vtracks, this_arg);
       sit++;
 
-    } else if ((this_arg == "-s") || (this_arg == "--stracks")) {
+    } else if ((this_arg == "-s") || (this_arg == "--stracks") || (this_arg == "--sub-tracks") || (this_arg == "--subtitle-tracks")) {
       if (no_next_arg)
         mxerror(boost::format(Y("'%1%' lacks the track number(s).\n")) % this_arg);
 
       parse_arg_tracks(next_arg, ti->stracks, this_arg);
       sit++;
 
-    } else if ((this_arg == "-b") || (this_arg == "--btracks")) {
+    } else if ((this_arg == "-b") || (this_arg == "--btracks") || (this_arg == "--button-tracks")) {
       if (no_next_arg)
         mxerror(boost::format(Y("'%1%' lacks the track number(s).\n")) % this_arg);
 
       parse_arg_tracks(next_arg, ti->btracks, this_arg);
       sit++;
 
+    } else if ((this_arg == "--track-tags")) {
+      if (no_next_arg)
+        mxerror(boost::format(Y("'%1%' lacks the track number(s).\n")) % this_arg);
+
+      parse_arg_tracks(next_arg, ti->track_tags, this_arg);
+      sit++;
+
     } else if ((this_arg == "-f") || (this_arg == "--fourcc")) {
       if (no_next_arg)
         mxerror(boost::format(Y("'%1%' lacks the FourCC.\n")) % this_arg);
@@ -2092,10 +2066,7 @@ parse_args(vector<string> args) {
       parse_arg_nalu_size_length(next_arg, *ti);
       sit++;
 
-    } else if (this_arg == "--print-malloc-report")
-      s_print_malloc_report = true;
-
-    else if (this_arg.length() == 0)
+    } else if (this_arg.length() == 0)
       mxerror(Y("An empty file name is not valid.\n"));
 
     // The argument is an input file.
@@ -2146,6 +2117,8 @@ parse_args(vector<string> args) {
         delete ti;
 
       ti = new track_info_c;
+      g_chapter_charset.clear();
+      g_chapter_language.clear();
     }
   }
 
@@ -2200,8 +2173,5 @@ main(int argc,
 
   cleanup();
 
-  if (s_print_malloc_report)
-    dump_malloc_report();
-
   mxexit();
 }
