@@ -12,11 +12,12 @@
    Modified by Steve Lhomme <steve.lhomme@free.fr>.
 */
 
-#include "os.h"
+#include "common/os.h"
 
 #include <algorithm>
 #include <map>
 #include <string>
+#include <typeinfo>
 
 #include <matroska/KaxContentEncoding.h>
 #include <matroska/KaxTag.h>
@@ -25,18 +26,20 @@
 #include <matroska/KaxTrackAudio.h>
 #include <matroska/KaxTrackVideo.h>
 
-#include "base64.h"
-#include "common.h"
-#include "commonebml.h"
-#include "compression.h"
-#include "hacks.h"
-#include "mkvmerge.h"
-#include "output_control.h"
-#include "pr_generic.h"
-#include "tagparser.h"
-#include "tag_common.h"
+#include "common/base64.h"
+#include "common/common.h"
+#include "common/compression.h"
+#include "common/ebml.h"
+#include "common/hacks.h"
+#include "common/math.h"
+#include "common/strings/formatting.h"
+#include "common/tags/tags.h"
+#include "common/tags/parser.h"
+#include "common/unique_numbers.h"
+#include "merge/mkvmerge.h"
+#include "merge/output_control.h"
+#include "merge/pr_generic.h"
 
-using namespace std;
 
 #define TRACK_TYPE_TO_DEFTRACK_TYPE(track_type)      \
   (  track_audio == track_type ? DEFTRACK_TYPE_AUDIO \
@@ -50,12 +53,13 @@ packet_t::~packet_t() {
 
 // ---------------------------------------------------------------------
 
-vector<generic_packetizer_c *> ptzrs_in_header_order;
+std::vector<generic_packetizer_c *> ptzrs_in_header_order;
 
 generic_packetizer_c::generic_packetizer_c(generic_reader_c *p_reader,
                                            track_info_c &p_ti)
   throw(error_c)
-  : next_packet_wo_assigned_timecode(0)
+  : m_num_packets(0)
+  , next_packet_wo_assigned_timecode(0)
   , m_free_refs(-1)
   , m_next_free_refs(-1)
   , enqueued_bytes(0)
@@ -103,6 +107,9 @@ generic_packetizer_c::generic_packetizer_c(generic_reader_c *p_reader,
   if (0 == ti.tcsync.denominator)
     ti.tcsync.denominator = 1;
 
+  // Let's see if the user specified "reset timecodes" for this track.
+  ti.reset_timecodes = map_has_key(ti.reset_timecodes_specs, ti.id) || map_has_key(ti.reset_timecodes_specs, -1);
+
   // Let's see if the user has specified which cues he wants for this track.
   if (map_has_key(ti.cue_creations, ti.id))
     ti.cues = ti.cue_creations[ti.id];
@@ -139,6 +146,7 @@ generic_packetizer_c::generic_packetizer_c(generic_reader_c *p_reader,
   else if (map_has_key(ti.all_tags, -1))
     ti.tags_file_name = ti.all_tags[-1];
   if (ti.tags_file_name != "") {
+    delete ti.tags;
     ti.tags = new KaxTags;
     parse_xml_tags(ti.tags_file_name, ti.tags);
   }
@@ -233,6 +241,12 @@ generic_packetizer_c::generic_packetizer_c(generic_reader_c *p_reader,
     hserialno = create_track_number(reader, ti.id);
 
   timecode_factory = timecode_factory_c::create(ti.ext_timecodes, ti.fname, ti.id);
+
+  // If no external timecode file but a default duration has been
+  // given then create a simple timecode factory that generates the
+  // timecodes for the given FPS.
+  if (!timecode_factory.is_set() && (-1 != htrack_default_duration))
+    timecode_factory = timecode_factory_c::create_fps_factory(htrack_default_duration, ti.fname, ti.id);
 }
 
 generic_packetizer_c::~generic_packetizer_c() {
@@ -325,17 +339,20 @@ generic_packetizer_c::set_track_type(int type,
 
   else if (TFA_AUTOMATIC != tfa_mode)
     timecode_factory_application_mode = tfa_mode;
+
+  if (timecode_factory.is_set() && (track_video != type) && (track_audio != type))
+    timecode_factory->set_preserve_duration(true);
 }
 
 void
-generic_packetizer_c::set_track_name(const string &name) {
+generic_packetizer_c::set_track_name(const std::string &name) {
   ti.track_name = name;
   if ((NULL != track_entry) && !name.empty())
     GetChildAs<KaxTrackName, EbmlUnicodeString>(track_entry) = cstrutf8_to_UTFstring(ti.track_name);
 }
 
 void
-generic_packetizer_c::set_codec_id(const string &id) {
+generic_packetizer_c::set_codec_id(const std::string &id) {
   hcodec_id = id;
   if ((NULL != track_entry) && !id.empty())
     GetChildAs<KaxCodecID, EbmlString>(track_entry) = hcodec_id;
@@ -476,7 +493,7 @@ generic_packetizer_c::set_as_default_track(int type,
 }
 
 void
-generic_packetizer_c::set_language(const string &language) {
+generic_packetizer_c::set_language(const std::string &language) {
   ti.language = language;
   if (NULL != track_entry)
     GetChildAs<KaxTrackLanguage, EbmlString>(track_entry) = ti.language;
@@ -512,8 +529,10 @@ generic_packetizer_c::set_stereo_mode(stereo_mode_e stereo_mode) {
 
 void
 generic_packetizer_c::set_headers() {
-  if (0 < connected_to)
+  if (0 < connected_to) {
+    mxerror(boost::format("generic_packetizer_c::set_headers(): connected_to > 0 (type: %1%). %2%\n") % typeid(*this).name() % BUGMSG);
     return;
+  }
 
   bool found = false;
   int idx;
@@ -557,7 +576,7 @@ generic_packetizer_c::set_headers() {
   if (-1 != htrack_max_add_block_ids)
     GetChildAs<KaxMaxBlockAdditionID, EbmlUInteger>(track_entry) = htrack_max_add_block_ids;
 
-  if (NULL != timecode_factory.get())
+  if (timecode_factory.is_set())
     htrack_default_duration = (int64_t)timecode_factory->get_default_duration(htrack_default_duration);
   if (-1.0 != htrack_default_duration)
     GetChildAs<KaxTrackDefaultDuration, EbmlUInteger>(track_entry) = htrack_default_duration;
@@ -686,6 +705,11 @@ generic_packetizer_c::fix_headers() {
 
 void
 generic_packetizer_c::add_packet(packet_cptr pack) {
+  if ((0 == m_num_packets) && ti.reset_timecodes)
+    ti.tcsync.displacement = -pack->timecode;
+
+  ++m_num_packets;
+
   if (NULL == reader->ptzr_first_packet)
     reader->ptzr_first_packet = this;
 
@@ -694,7 +718,7 @@ generic_packetizer_c::add_packet(packet_cptr pack) {
       && (pack->data_adds.size()  > htrack_max_add_block_ids))
     pack->data_adds.resize(htrack_max_add_block_ids);
 
-  if (NULL != compressor.get()) {
+  if (compressor.is_set()) {
     try {
       compressor->compress(pack->data);
       int i;
@@ -754,7 +778,7 @@ generic_packetizer_c::add_packet2(packet_cptr pack) {
   // 'timecode < safety_last_timecode' may only occur for B frames. In this
   // case we have the coding order, e.g. IPB1B2 and the timecodes
   // I: 0, P: 120, B1: 40, B2: 80.
-  if (!relaxed_timecode_checking && (pack->timecode < safety_last_timecode) && (0 > pack->fref)) {
+  if (!relaxed_timecode_checking && (pack->timecode < safety_last_timecode) && (0 > pack->fref) && hack_engaged(ENGAGE_ENABLE_TIMECODE_WARNING)) {
     if (track_audio == htrack_type) {
       int64_t needed_timecode_offset  = safety_last_timecode + safety_last_duration - pack->timecode;
       correction_timecode_offset     += needed_timecode_offset;
@@ -775,7 +799,7 @@ generic_packetizer_c::add_packet2(packet_cptr pack) {
                                  "In this case you should contact the author Moritz Bunkus <moritz@bunkus.org>.\n"))
                  % ((needed_timecode_offset + 500000) / 1000000));
 
-    } else if (hack_engaged(ENGAGE_ENABLE_TIMECODE_WARNING))
+    } else
       mxwarn_tid(ti.fname, ti.id,
                  boost::format(Y("pr_generic.cpp/generic_packetizer_c::add_packet(): timecode < last_timecode (%1% < %2%). %3%\n"))
                  % format_timecode(pack->timecode) % format_timecode(safety_last_timecode) % BUGMSG);
@@ -786,7 +810,7 @@ generic_packetizer_c::add_packet2(packet_cptr pack) {
   pack->timecode_before_factory = pack->timecode;
 
   packet_queue.push_back(pack);
-  if ((NULL == timecode_factory.get()) || (TFA_IMMEDIATE == timecode_factory_application_mode))
+  if (!timecode_factory.is_set() || (TFA_IMMEDIATE == timecode_factory_application_mode))
     apply_factory_once(pack);
   else
     apply_factory();
@@ -794,7 +818,7 @@ generic_packetizer_c::add_packet2(packet_cptr pack) {
 
 void
 generic_packetizer_c::process_deferred_packets() {
-  deque<packet_cptr>::iterator packet;
+  std::deque<packet_cptr>::iterator packet;
 
   mxforeach(packet, deferred_packets)
     add_packet2(*packet);
@@ -820,7 +844,7 @@ generic_packetizer_c::get_packet() {
 
 void
 generic_packetizer_c::apply_factory_once(packet_cptr &packet) {
-  if (NULL == timecode_factory.get()) {
+  if (!timecode_factory.is_set()) {
     packet->assigned_timecode = packet->timecode;
     packet->gap_following     = false;
   } else
@@ -887,7 +911,7 @@ generic_packetizer_c::apply_factory_short_queueing(packet_cptr_di &p_start) {
 
 struct packet_sorter_t {
   int m_index;
-  static deque<packet_cptr> *m_packet_queue;
+  static std::deque<packet_cptr> *m_packet_queue;
 
   packet_sorter_t(int index)
     : m_index(index)
@@ -899,7 +923,7 @@ struct packet_sorter_t {
   }
 };
 
-deque<packet_cptr> *packet_sorter_t::m_packet_queue = NULL;
+std::deque<packet_cptr> *packet_sorter_t::m_packet_queue = NULL;
 
 void
 generic_packetizer_c::apply_factory_full_queueing(packet_cptr_di &p_start) {
@@ -919,7 +943,7 @@ generic_packetizer_c::apply_factory_full_queueing(packet_cptr_di &p_start) {
 
     // Now sort the frames by their timecode as the factory has to be
     // applied to the packets in the same order as they're timestamped.
-    vector<packet_sorter_t> sorter;
+    std::vector<packet_sorter_t> sorter;
     bool needs_sorting        = false;
     int64_t previous_timecode = 0;
     int                       i = distance(packet_queue.begin(), p_start);
@@ -968,17 +992,15 @@ generic_packetizer_c::handle_avi_audio_sync(int64_t num_bytes,
      duration = num_bytes * 1000000000 / ti.avi_avg_bytes_per_sec;
 
   else {
-    double samples = 0.0;
-
+    int num_blocks = 0;
     int i;
     for (i = 0; (ti.avi_block_sizes.size() > i) && (0 < num_bytes); ++i) {
-      int64_t block_size = ti.avi_block_sizes[i];
-      int cur_bytes      = num_bytes < block_size ? num_bytes : block_size;
-
-      samples   += (double)ti.avi_samples_per_chunk * cur_bytes / ti.avi_block_align;
-      num_bytes -= cur_bytes;
+      int64_t block_size  = ti.avi_block_sizes[i];
+      num_blocks         += (block_size + ti.avi_block_align - 1) / ti.avi_block_align;
+      num_bytes          -= std::min(num_bytes, block_size);
     }
-    duration = (int64_t)(samples * 1000000000 / ti.avi_samples_per_sec);
+
+    duration = static_cast<int64_t>(num_blocks * 1000000000ll * static_cast<double>(ti.avi_samples_per_chunk) / static_cast<double>(ti.avi_sample_scale));
   }
 
   enable_avi_audio_sync(false);
@@ -1020,10 +1042,7 @@ generic_packetizer_c::set_displacement_maybe(int64_t displacement) {
 
 bool
 generic_packetizer_c::contains_gap() {
-  if (NULL != timecode_factory.get())
-    return timecode_factory->contains_gap();
-  else
-    return false;
+  return timecode_factory.is_set() ? timecode_factory->contains_gap() : false;
 }
 
 void
@@ -1035,7 +1054,7 @@ generic_packetizer_c::flush() {
 //--------------------------------------------------------------------
 
 #define add_all_requested_track_ids(type, container)                                              \
-  for (map<int64_t, type>::const_iterator i = ti.container.begin(); ti.container.end() != i; ++i) \
+  for (std::map<int64_t, type>::const_iterator i = ti.container.begin(); ti.container.end() != i; ++i) \
     add_requested_track_id(i->first);
 
 #define add_all_requested_track_ids2(container) \
@@ -1058,18 +1077,19 @@ generic_reader_c::generic_reader_c(track_info_c &_ti)
   add_all_requested_track_ids2(vtracks);
   add_all_requested_track_ids2(stracks);
   add_all_requested_track_ids2(btracks);
-  add_all_requested_track_ids(string, all_fourccs);
+  add_all_requested_track_ids2(track_tags);
+  add_all_requested_track_ids(std::string, all_fourccs);
   add_all_requested_track_ids(display_properties_t, display_properties);
   add_all_requested_track_ids(timecode_sync_t, timecode_syncs);
   add_all_requested_track_ids(cue_strategy_e, cue_creations);
   add_all_requested_track_ids(bool, default_track_flags);
-  add_all_requested_track_ids(string, languages);
-  add_all_requested_track_ids(string, sub_charsets);
-  add_all_requested_track_ids(string, all_tags);
+  add_all_requested_track_ids(std::string, languages);
+  add_all_requested_track_ids(std::string, sub_charsets);
+  add_all_requested_track_ids(std::string, all_tags);
   add_all_requested_track_ids(bool, all_aac_is_sbr);
   add_all_requested_track_ids(compression_method_e, compression_list);
-  add_all_requested_track_ids(string, track_names);
-  add_all_requested_track_ids(string, all_ext_timecodes);
+  add_all_requested_track_ids(std::string, track_names);
+  add_all_requested_track_ids(std::string, all_ext_timecodes);
   add_all_requested_track_ids(pixel_crop_t, pixel_crop_list);
 }
 
@@ -1095,7 +1115,7 @@ generic_reader_c::read_all() {
 bool
 generic_reader_c::demuxing_requested(char type,
                                      int64_t id) {
-  vector<int64_t> *tracks = NULL;
+  std::vector<int64_t> *tracks = NULL;
 
   if ('v' == type) {
     if (ti.no_video)
@@ -1117,6 +1137,11 @@ generic_reader_c::demuxing_requested(char type,
       return false;
     tracks = &ti.btracks;
 
+  } else if ('T' == type) {
+    if (ti.no_track_tags)
+      return false;
+    tracks = &ti.track_tags;
+
   } else
     mxerror(boost::format(Y("pr_generic.cpp/generic_reader_c::demuxing_requested(): Invalid track type %1%.")) % type);
 
@@ -1162,14 +1187,14 @@ void
 generic_reader_c::set_timecode_offset(int64_t offset) {
   max_timecode_seen = offset;
 
-  vector<generic_packetizer_c *>::const_iterator it;
+  std::vector<generic_packetizer_c *>::const_iterator it;
   mxforeach(it, reader_packetizers)
     (*it)->correction_timecode_offset = offset;
 }
 
 void
 generic_reader_c::set_headers() {
-  vector<generic_packetizer_c *>::const_iterator it;
+  std::vector<generic_packetizer_c *>::const_iterator it;
 
   mxforeach(it, reader_packetizers)
     (*it)->set_headers();
@@ -1177,7 +1202,7 @@ generic_reader_c::set_headers() {
 
 void
 generic_reader_c::set_headers_for_track(int64_t tid) {
-  vector<generic_packetizer_c *>::const_iterator it;
+  std::vector<generic_packetizer_c *>::const_iterator it;
 
   mxforeach(it, reader_packetizers)
     if ((*it)->ti.id == tid) {
@@ -1189,8 +1214,6 @@ generic_reader_c::set_headers_for_track(int64_t tid) {
 void
 generic_reader_c::check_track_ids_and_packetizers() {
   add_available_track_ids();
-  if (reader_packetizers.size() == 0)
-    mxwarn_fn(ti.fname, Y("No tracks will be copied from this file. This usually indicates a mistake in the command line.\n"));
 
   int r;
   for (r = 0; requested_track_ids.size() > r; ++r) {
@@ -1230,7 +1253,7 @@ int64_t
 generic_reader_c::get_queued_bytes() {
   int64_t  bytes = 0;
 
-  vector<generic_packetizer_c *>::const_iterator it;
+  std::vector<generic_packetizer_c *>::const_iterator it;
   mxforeach(it, reader_packetizers)
     bytes += (*it)->get_queued_bytes();
 
@@ -1239,15 +1262,15 @@ generic_reader_c::get_queued_bytes() {
 
 void
 generic_reader_c::flush_packetizers() {
-  vector<generic_packetizer_c *>::const_iterator it;
+  std::vector<generic_packetizer_c *>::const_iterator it;
 
   mxforeach(it, reader_packetizers)
     (*it)->flush();
 }
 
 void
-generic_reader_c::id_result_container_unsupported(const string &filename,
-                                                  const string &info) {
+generic_reader_c::id_result_container_unsupported(const std::string &filename,
+                                                  const std::string &info) {
   if (g_identifying) {
     if (g_identify_for_mmg)
       mxinfo(boost::format("File '%1%': unsupported container: %2%\n") % filename % info);
@@ -1260,8 +1283,8 @@ generic_reader_c::id_result_container_unsupported(const string &filename,
 }
 
 void
-generic_reader_c::id_result_container(const string &info,
-                                      const string &verbose_info) {
+generic_reader_c::id_result_container(const std::string &info,
+                                      const std::string &verbose_info) {
   id_results_container.info = info;
   id_results_container.verbose_info.clear();
   if (!verbose_info.empty())
@@ -1269,17 +1292,17 @@ generic_reader_c::id_result_container(const string &info,
 }
 
 void
-generic_reader_c::id_result_container(const string &info,
-                                      const vector<string> &verbose_info) {
+generic_reader_c::id_result_container(const std::string &info,
+                                      const std::vector<std::string> &verbose_info) {
   id_results_container.info         = info;
   id_results_container.verbose_info = verbose_info;
 }
 
 void
 generic_reader_c::id_result_track(int64_t track_id,
-                                  const string &type,
-                                  const string &info,
-                                  const string &verbose_info) {
+                                  const std::string &type,
+                                  const std::string &info,
+                                  const std::string &verbose_info) {
   id_result_t result(track_id, type, info, empty_string, 0);
   if (!verbose_info.empty())
     result.verbose_info.push_back(verbose_info);
@@ -1288,9 +1311,9 @@ generic_reader_c::id_result_track(int64_t track_id,
 
 void
 generic_reader_c::id_result_track(int64_t track_id,
-                                  const string &type,
-                                  const string &info,
-                                  const vector<string> &verbose_info) {
+                                  const std::string &type,
+                                  const std::string &info,
+                                  const std::vector<std::string> &verbose_info) {
   id_result_t result(track_id, type, info, empty_string, 0);
   result.verbose_info = verbose_info;
   id_results_tracks.push_back(result);
@@ -1298,17 +1321,30 @@ generic_reader_c::id_result_track(int64_t track_id,
 
 void
 generic_reader_c::id_result_attachment(int64_t attachment_id,
-                                       const string &type,
+                                       const std::string &type,
                                        int size,
-                                       const string &file_name,
-                                       const string &description) {
+                                       const std::string &file_name,
+                                       const std::string &description) {
   id_result_t result(attachment_id, type, file_name, description, size);
   id_results_attachments.push_back(result);
 }
 
+void
+generic_reader_c::id_result_chapters(int num_entries) {
+  id_result_t result(0, ID_RESULT_CHAPTERS, empty_string, empty_string, num_entries);
+  id_results_chapters.push_back(result);
+}
+
+void
+generic_reader_c::id_result_tags(int64_t track_id,
+                                 int num_entries) {
+  id_result_t result(track_id, ID_RESULT_TAGS, empty_string, empty_string, num_entries);
+  id_results_tags.push_back(result);
+}
+
 void
 generic_reader_c::display_identification_results() {
-  string format_file, format_track, format_attachment, format_att_description, format_att_file_name;
+  std::string format_file, format_track, format_attachment, format_att_description, format_att_file_name, format_chapters, format_tags_global, format_tags_track;
 
   if (g_identify_for_mmg) {
     format_file            =   "File '%1%': container: %2%";
@@ -1316,6 +1352,9 @@ generic_reader_c::display_identification_results() {
     format_attachment      =   "Attachment ID %1%: type \"%2%\", size %3% bytes";
     format_att_description =   ", description \"%1%\"";
     format_att_file_name   =   ", file name \"%1%\"";
+    format_chapters        =   "Chapters: %1% entries";
+    format_tags_global     =   "Global tags: %1% entries";
+    format_tags_track      =   "Tags for track ID %1%: %2% entries";
 
   } else {
     format_file            = Y("File '%1%': container: %2%");
@@ -1323,6 +1362,9 @@ generic_reader_c::display_identification_results() {
     format_attachment      = Y("Attachment ID %1%: type '%2%', size %3% bytes");
     format_att_description = Y(", description '%1%'");
     format_att_file_name   = Y(", file name '%1%'");
+    format_chapters        = Y("Chapters: %1% entries");
+    format_tags_global     = Y("Global tags: %1% entries");
+    format_tags_track      = Y("Tags for track ID %1%: %2% entries");
   }
 
   mxinfo(boost::format(format_file) % ti.fname % id_results_container.info);
@@ -1333,7 +1375,7 @@ generic_reader_c::display_identification_results() {
   mxinfo("\n");
 
   int i;
-  for (i = 0; i < id_results_tracks.size(); ++i) {
+  for (i = 0; id_results_tracks.size() > i; ++i) {
     id_result_t &result = id_results_tracks[i];
 
     mxinfo(boost::format(format_track) % result.id % result.type % result.info);
@@ -1344,7 +1386,7 @@ generic_reader_c::display_identification_results() {
     mxinfo("\n");
   }
 
-  for (i = 0; i < id_results_attachments.size(); ++i) {
+  for (i = 0; id_results_attachments.size() > i; ++i) {
     id_result_t &result = id_results_attachments[i];
 
     mxinfo(boost::format(format_attachment) % result.id % id_escape_string(result.type) % result.size);
@@ -1357,6 +1399,22 @@ generic_reader_c::display_identification_results() {
 
     mxinfo("\n");
   }
+
+  for (i = 0; id_results_chapters.size() > i; ++i) {
+    mxinfo(boost::format(format_chapters) % id_results_chapters[i].size);
+    mxinfo("\n");
+  }
+
+  for (i = 0; id_results_tags.size() > i; ++i) {
+    id_result_t &result = id_results_tags[i];
+
+    if (ID_RESULT_GLOBAL_TAGS_ID == result.id)
+      mxinfo(boost::format(format_tags_global) % result.size);
+    else
+      mxinfo(boost::format(format_tags_track) % result.id % result.size);
+
+    mxinfo("\n");
+  }
 }
 
 std::string
@@ -1374,6 +1432,7 @@ track_info_c::track_info_c()
   , no_video(false)
   , no_subs(false)
   , no_buttons(false)
+  , no_track_tags(false)
   , private_data(NULL)
   , private_size(0)
   , aspect_ratio(0.0)
@@ -1382,6 +1441,7 @@ track_info_c::track_info_c()
   , aspect_ratio_given(false)
   , aspect_ratio_is_factor(false)
   , display_dimensions_given(false)
+  , reset_timecodes(false)
   , cues(CUE_STRATEGY_UNSPECIFIED)
   , default_track(boost::logic::indeterminate)
   , forced_track(boost::logic::indeterminate)
@@ -1392,11 +1452,12 @@ track_info_c::track_info_c()
   , nalu_size_length(0)
   , no_chapters(false)
   , no_attachments(false)
-  , no_tags(false)
+  , no_global_tags(false)
   , avi_block_align(0)
   , avi_samples_per_sec(0)
   , avi_avg_bytes_per_sec(0)
   , avi_samples_per_chunk(0)
+  , avi_sample_scale(0)
   , avi_audio_sync_enabled(false)
 {
   memset(&pixel_cropping, 0, sizeof(pixel_crop_t));
@@ -1424,10 +1485,12 @@ track_info_c::operator =(const track_info_c &src) {
   no_video                   = src.no_video;
   no_subs                    = src.no_subs;
   no_buttons                 = src.no_buttons;
+  no_track_tags              = src.no_track_tags;
   atracks                    = src.atracks;
   btracks                    = src.btracks;
   stracks                    = src.stracks;
   vtracks                    = src.vtracks;
+  track_tags                 = src.track_tags;
 
   private_size               = src.private_size;
   private_data               = (unsigned char *)safememdup(src.private_data, private_size);
@@ -1443,6 +1506,9 @@ track_info_c::operator =(const track_info_c &src) {
   timecode_syncs             = src.timecode_syncs;
   memcpy(&tcsync, &src.tcsync, sizeof(timecode_sync_t));
 
+  reset_timecodes_specs      = src.reset_timecodes_specs;
+  reset_timecodes            = src.reset_timecodes;
+
   cue_creations              = src.cue_creations;
   cues                       = src.cues;
 
@@ -1487,14 +1553,16 @@ track_info_c::operator =(const track_info_c &src) {
 
   no_chapters                = src.no_chapters;
   no_attachments             = src.no_attachments;
-  no_tags                    = src.no_tags;
+  no_global_tags             = src.no_global_tags;
 
   chapter_charset            = src.chapter_charset;
+  chapter_language           = src.chapter_language;
 
   avi_block_align            = src.avi_block_align;
   avi_samples_per_sec        = src.avi_samples_per_sec;
   avi_avg_bytes_per_sec      = src.avi_avg_bytes_per_sec;
   avi_samples_per_chunk      = src.avi_samples_per_chunk;
+  avi_sample_scale           = src.avi_sample_scale;
   avi_block_sizes.clear();
   avi_audio_sync_enabled     = false;
 
