@@ -11,7 +11,7 @@
    Written by Moritz Bunkus <moritz@bunkus.org>.
 */
 
-#include "os.h"
+#include "common/os.h"
 
 #include <errno.h>
 #ifdef HAVE_UNISTD_H
@@ -34,16 +34,16 @@
 #include <windows.h>
 #endif
 
-#include "common.h"
-#include "jobs.h"
-#include "mmg.h"
-#include "mmg_dialog.h"
+#include "common/common.h"
+#include "mmg/jobs.h"
+#include "mmg/mmg.h"
+#include "mmg/mmg_dialog.h"
 
 #define JOB_LOG_DIALOG_WIDTH 600
 #define JOB_RUN_DIALOG_WIDTH 500
 
 job_run_dialog::job_run_dialog(wxWindow *parent,
-                               vector<int> &n_jobs_to_start)
+                               std::vector<int> &n_jobs_to_start)
   : wxDialog(NULL, -1, Z("mkvmerge is running"), wxDefaultPosition, wxDefaultSize, wxDEFAULT_DIALOG_STYLE | wxRESIZE_BORDER | wxMINIMIZE_BOX | wxMAXIMIZE_BOX)
   , t_update(new wxTimer(this, 1))
   , out(NULL)
@@ -158,7 +158,7 @@ job_run_dialog::start_next_job() {
     if ((*arg_list)[i].Length() == 0)
       opt_file->Write(wxT("#EMPTY#"));
     else {
-      string arg_utf8 = to_utf8((*arg_list)[i]);
+      std::string arg_utf8 = wxMB((*arg_list)[i]);
       opt_file->Write(arg_utf8.c_str(), arg_utf8.length());
     }
     opt_file->Write(wxT("\n"));
@@ -167,7 +167,12 @@ job_run_dialog::start_next_job() {
 
   process = new wxProcess(this, 1);
   process->Redirect();
-  pid = wxExecute((*arg_list)[0] + wxT(" \"@") + opt_file_name + wxT("\""), wxEXEC_ASYNC, process);
+  wxString command_line = wxString::Format(wxT("\"%s\" \"@%s\""), (*arg_list)[0].c_str(), opt_file_name.c_str());
+  pid = wxExecute(command_line, wxEXEC_ASYNC, process);
+  if (0 == pid) {
+    wxLogError(wxT("Execution of '%s' failed."), command_line.c_str());
+    return;
+  }
   out = process->GetInputStream();
 
   *jobs[ndx].log        = wxEmptyString;
@@ -473,7 +478,7 @@ job_dialog::enable_buttons(bool enable,
 void
 job_dialog::on_start(wxCommandEvent &evt) {
   int i;
-  vector<int> jobs_to_start;
+  std::vector<int> jobs_to_start;
 
   for (i = 0; jobs.size() > i; ++i)
     if (JOBS_PENDING == jobs[i].status)
@@ -486,7 +491,7 @@ job_dialog::on_start(wxCommandEvent &evt) {
 void
 job_dialog::on_start_selected(wxCommandEvent &evt) {
   int i;
-  vector<int> jobs_to_start;
+  std::vector<int> jobs_to_start;
 
   for (i = 0; jobs.size() > i; ++i)
     if (lv_jobs->IsSelected(i))
@@ -499,7 +504,7 @@ job_dialog::on_start_selected(wxCommandEvent &evt) {
 void
 job_dialog::on_delete(wxCommandEvent &evt) {
   int i;
-  vector<bool> selected;
+  std::vector<bool> selected;
 
   for (i = 0; lv_jobs->GetItemCount() > i; ++i)
     selected.push_back(lv_jobs->IsSelected(i));
@@ -548,7 +553,7 @@ job_dialog::swap_rows(int lower,
 
 void
 job_dialog::on_up(wxCommandEvent &evt) {
-  vector<bool> selected;
+  std::vector<bool> selected;
 
   bool first = true;
   int i      = 0;
@@ -573,7 +578,7 @@ job_dialog::on_up(wxCommandEvent &evt) {
 
 void
 job_dialog::on_down(wxCommandEvent &evt) {
-  vector<bool> selected;
+  std::vector<bool> selected;
 
   bool first = true;
   int i      = lv_jobs->GetItemCount() - 1;
@@ -663,7 +668,7 @@ job_dialog::on_item_selected(wxListEvent &evt) {
 }
 
 void
-job_dialog::start_jobs(vector<int> &jobs_to_start) {
+job_dialog::start_jobs(std::vector<int> &jobs_to_start) {
   wxString temp_settings = wxGetCwd() + wxT("/jobs/temp.mmg");
   mdlg->save(temp_settings, true);
 
@@ -686,6 +691,11 @@ job_dialog::start_jobs(vector<int> &jobs_to_start) {
 
 void
 job_dialog::on_key_pressed(wxKeyEvent &evt) {
+  if (WXK_RETURN == evt.GetKeyCode()) {
+    EndModal(0);
+    return;
+  }
+
   if (1 != evt.GetKeyCode()) {  // Ctrl-A
     evt.Skip(true);
     return;
@@ -705,7 +715,9 @@ jobdlg_list_view::jobdlg_list_view(wxWindow *parent,
 
 void
 jobdlg_list_view::on_key_pressed(wxKeyEvent &evt) {
-  if (1 == evt.GetKeyCode())    // Ctrl-A
+  int key_code = evt.GetKeyCode();
+  if (   (1 == key_code)        // Ctrl-A
+      || (WXK_RETURN == key_code))
     ((job_dialog *)GetParent())->on_key_pressed(evt);
   else
     evt.Skip(true);
