@@ -11,7 +11,7 @@
    Written by Moritz Bunkus <moritz@bunkus.org>.
 */
 
-#include "os.h"
+#include "common/os.h"
 
 #include <errno.h>
 #ifdef HAVE_UNISTD_H
@@ -34,10 +34,10 @@
 #include <windows.h>
 #endif
 
-#include "common.h"
-#include "mmg.h"
-#include "mmg_dialog.h"
-#include "mux_dialog.h"
+#include "common/common.h"
+#include "mmg/mmg.h"
+#include "mmg/mmg_dialog.h"
+#include "mmg/mux_dialog.h"
 
 mux_dialog::mux_dialog(wxWindow *parent):
   wxDialog(parent, -1, Z("mkvmerge is running"), wxDefaultPosition,
@@ -46,9 +46,11 @@ mux_dialog::mux_dialog(wxWindow *parent):
 #else
            wxSize(700, 520),
 #endif
-           wxDEFAULT_FRAME_STYLE) {
+           wxDEFAULT_FRAME_STYLE)
+  , m_exit_code(0)
+{
   char c;
-  string arg_utf8, line;
+  std::string arg_utf8, line;
   long value;
   wxString wx_line, tmp;
   wxInputStream *out;
@@ -118,15 +120,20 @@ mux_dialog::mux_dialog(wxWindow *parent):
     if ((*arg_list)[i].Length() == 0)
       opt_file->Write(wxT("#EMPTY#"));
     else {
-      arg_utf8 = to_utf8((*arg_list)[i]);
+      arg_utf8 = wxMB((*arg_list)[i]);
       opt_file->Write(arg_utf8.c_str(), arg_utf8.length());
     }
     opt_file->Write(wxT("\n"));
   }
   delete opt_file;
 
-  pid = wxExecute((*arg_list)[0] + wxT(" \"@") + opt_file_name + wxT("\""),
-                  wxEXEC_ASYNC, process);
+  wxString command_line = wxString::Format(wxT("\"%s\" \"@%s\""), (*arg_list)[0].c_str(), opt_file_name.c_str());
+  pid = wxExecute(command_line, wxEXEC_ASYNC, process);
+  if (0 == pid) {
+    wxLogError(wxT("Execution of '%s' failed."), command_line.c_str());
+    done();
+    return;
+  }
   out = process->GetInputStream();
 
   line = "";
@@ -223,7 +230,7 @@ mux_dialog::on_abort(wxCommandEvent &evt) {
 
 void
 mux_dialog::on_close(wxCloseEvent &evt) {
-  mdlg->muxing_has_finished();
+  mdlg->muxing_has_finished(m_exit_code);
   delete m_window_disabler;
   Destroy();
 }
@@ -248,6 +255,8 @@ mux_process::OnTerminate(int pid,
   if (NULL == dlg)
     return;
 
+  dlg->set_exit_code(status);
+
   wxString format;
   if ((status != 0) && (status != 1))
     format = Z("mkvmerge FAILED with a return code of %d. %s\n");
