@@ -11,18 +11,13 @@
    Written by Moritz Bunkus <moritz@bunkus.org>.
 */
 
-#include "os.h"
+#include "common/os.h"
 
-#include <stdlib.h>
-#include <stdio.h>
-#include <string.h>
-#include <errno.h>
-
-#include "ac3_common.h"
-#include "matroska.h"
-#include "output_control.h"
-#include "pr_generic.h"
-#include "p_ac3.h"
+#include "common/ac3.h"
+#include "common/matroska.h"
+#include "merge/output_control.h"
+#include "merge/pr_generic.h"
+#include "output/p_ac3.h"
 
 using namespace libmatroska;
 
@@ -36,15 +31,19 @@ ac3_packetizer_c::ac3_packetizer_c(generic_reader_c *p_reader,
   , m_bytes_output(0)
   , m_packetno(0)
   , m_bytes_skipped(0)
+  , m_last_timecode(-1)
+  , m_num_packets_same_tc(0)
   , m_samples_per_sec(samples_per_sec)
   , m_first_packet(true)
+  , m_s2tc(1536 * 1000000000ll, m_samples_per_sec)
+  , m_single_packet_duration(1 * m_s2tc)
 {
   memset(&m_first_ac3_header, 0, sizeof(ac3_header_t));
   m_first_ac3_header.bsid     = bsid;
   m_first_ac3_header.channels = channels;
 
   set_track_type(track_audio);
-  set_track_default_duration((int64_t)(1536000000000.0 / m_samples_per_sec));
+  set_track_default_duration(m_single_packet_duration);
   enable_avi_audio_sync(true);
 }
 
@@ -109,7 +108,7 @@ ac3_packetizer_c::get_ac3_packet(unsigned long *header,
 
 void
 ac3_packetizer_c::set_headers() {
-  string id = MKV_A_AC3;
+  std::string id = MKV_A_AC3;
 
   if (9 == m_first_ac3_header.bsid)
     id += "/BSID9";
@@ -131,13 +130,27 @@ ac3_packetizer_c::process(packet_cptr packet) {
   unsigned long header;
   ac3_header_t ac3header;
 
-  add_to_buffer(packet->data->get(), packet->data->get_size());
+  add_to_buffer(packet->data->get_buffer(), packet->data->get_size());
   while ((ac3_packet = get_ac3_packet(&header, &ac3header)) != NULL) {
     adjust_header_values(ac3header);
 
-    int64_t new_timecode = -1 == packet->timecode ? (int64_t)(1000000000.0 * m_packetno * ac3header.samples / m_samples_per_sec) : packet->timecode;
+    int64_t new_timecode;
 
-    add_packet(new packet_t(new memory_c(ac3_packet, ac3header.bytes, true), new_timecode, (int64_t)(1000000000.0 * ac3header.samples / m_samples_per_sec)));
+    if (-1 != packet->timecode) {
+      new_timecode = packet->timecode;
+      if (m_last_timecode == packet->timecode) {
+        m_num_packets_same_tc++;
+        new_timecode += m_num_packets_same_tc * m_s2tc;
+
+      } else {
+        m_last_timecode       = packet->timecode;
+        m_num_packets_same_tc = 0;
+      }
+
+    } else
+      new_timecode = m_packetno * m_s2tc;
+
+    add_packet(new packet_t(new memory_c(ac3_packet, ac3header.bytes, true), new_timecode, m_single_packet_duration));
     m_packetno++;
   }
 
@@ -156,15 +169,18 @@ ac3_packetizer_c::adjust_header_values(ac3_header_t &ac3_header) {
   if (16 == m_first_ac3_header.bsid)
     set_codec_id(MKV_A_EAC3);
 
-  if (1536 != m_first_ac3_header.samples)
-    set_track_default_duration((int64_t)(1000000000.0 * m_first_ac3_header.samples / m_samples_per_sec));
+  if (1536 != m_first_ac3_header.samples) {
+    m_s2tc.set(1000000000ll * m_first_ac3_header.samples, m_samples_per_sec);
+    m_single_packet_duration = 1 * m_s2tc;
+    set_track_default_duration(m_single_packet_duration);
+  }
 
   rerender_track_headers();
 }
 
 connection_result_e
 ac3_packetizer_c::can_connect_to(generic_packetizer_c *src,
-                                 string &error_message) {
+                                 std::string &error_message) {
   ac3_packetizer_c *asrc = dynamic_cast<ac3_packetizer_c *>(src);
   if (NULL == asrc)
     return CAN_CONNECT_NO_FORMAT;
