@@ -14,11 +14,11 @@
 #ifndef __P_AVC_H
 #define __P_AVC_H
 
-#include "os.h"
+#include "common/os.h"
 
-#include "common.h"
-#include "mpeg4_common.h"
-#include "pr_generic.h"
+#include "common/common.h"
+#include "common/mpeg4_p10.h"
+#include "merge/pr_generic.h"
 
 using namespace mpeg4::p10;
 
@@ -33,6 +33,7 @@ public:
   mpeg4_p10_es_video_packetizer_c(generic_reader_c *p_reader, track_info_c &p_ti, memory_cptr avcc, int width, int height);
 
   virtual int process(packet_cptr packet);
+  virtual void add_extra_data(memory_cptr data);
   virtual void set_headers();
 
   virtual void flush();
@@ -46,7 +47,7 @@ public:
   };
 
   virtual void connect(generic_packetizer_c *src, int64_t p_append_timecode_offset = -1);
-  virtual connection_result_e can_connect_to(generic_packetizer_c *src, string &error_message);
+  virtual connection_result_e can_connect_to(generic_packetizer_c *src, std::string &error_message);
 };
 
 #endif // __P_AVC_H
