@@ -14,11 +14,11 @@
 #ifndef __P_DIRAC_H
 #define __P_DIRAC_H
 
-#include "os.h"
+#include "common/os.h"
 
-#include "common.h"
-#include "pr_generic.h"
-#include "dirac_common.h"
+#include "common/common.h"
+#include "merge/pr_generic.h"
+#include "common/dirac.h"
 
 class dirac_video_packetizer_c: public generic_packetizer_c {
 protected:
@@ -41,7 +41,7 @@ public:
     return "video";
   };
 
-  virtual connection_result_e can_connect_to(generic_packetizer_c *src, string &error_message);
+  virtual connection_result_e can_connect_to(generic_packetizer_c *src, std::string &error_message);
 
 protected:
   virtual void flush_frames();
