@@ -11,17 +11,12 @@
    Written by Moritz Bunkus <moritz@bunkus.org>.
 */
 
-#include "os.h"
+#include "common/os.h"
 
-#include <stdlib.h>
-#include <stdio.h>
-#include <string.h>
-#include <errno.h>
-
-#include "common.h"
-#include "pr_generic.h"
-#include "p_passthrough.h"
-#include "matroska.h"
+#include "common/common.h"
+#include "common/matroska.h"
+#include "merge/pr_generic.h"
+#include "output/p_passthrough.h"
 
 using namespace libmatroska;
 
@@ -49,7 +44,7 @@ passthrough_packetizer_c::process(packet_cptr packet) {
 
 connection_result_e
 passthrough_packetizer_c::can_connect_to(generic_packetizer_c *src,
-                                         string &error_message) {
+                                         std::string &error_message) {
   passthrough_packetizer_c *psrc = dynamic_cast<passthrough_packetizer_c *>(src);
   if (NULL == psrc)
     return CAN_CONNECT_NO_FORMAT;
