@@ -11,16 +11,11 @@
    Written by Moritz Bunkus <moritz@bunkus.org>.
 */
 
-#include "os.h"
+#include "common/os.h"
 
-#include <ctype.h>
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-
-#include "matroska.h"
-#include "pr_generic.h"
-#include "p_realaudio.h"
+#include "common/matroska.h"
+#include "merge/pr_generic.h"
+#include "output/p_realaudio.h"
 
 using namespace libmatroska;
 
@@ -48,13 +43,13 @@ ra_packetizer_c::~ra_packetizer_c() {
 
 void
 ra_packetizer_c::set_headers() {
-  string codec_id = (boost::format("A_REAL/%1%%2%%3%%4%")
-                     % (char)(m_fourcc >> 24) % (char)((m_fourcc >> 16) & 0xff) % (char)((m_fourcc >> 8) & 0xff) % (char)(m_fourcc & 0xff)).str();
+  std::string codec_id = (boost::format("A_REAL/%1%%2%%3%%4%")
+                          % (char)(m_fourcc >> 24) % (char)((m_fourcc >> 16) & 0xff) % (char)((m_fourcc >> 8) & 0xff) % (char)(m_fourcc & 0xff)).str();
   set_codec_id(upcase(codec_id));
   set_audio_sampling_freq((float)m_samples_per_sec);
   set_audio_channels(m_channels);
   set_audio_bit_depth(m_bits_per_sample);
-  set_codec_private(m_private_data->get(), m_private_data->get_size());
+  set_codec_private(m_private_data->get_buffer(), m_private_data->get_size());
 
   generic_packetizer_c::set_headers();
   track_entry->EnableLacing(false);
@@ -69,7 +64,7 @@ ra_packetizer_c::process(packet_cptr packet) {
 
 connection_result_e
 ra_packetizer_c::can_connect_to(generic_packetizer_c *src,
-                                string &error_message) {
+                                std::string &error_message) {
   ra_packetizer_c *psrc = dynamic_cast<ra_packetizer_c *>(src);
   if (NULL == psrc)
     return CAN_CONNECT_NO_FORMAT;
