@@ -14,11 +14,11 @@
 #ifndef __P_REALAUDIO_H
 #define __P_REALAUDIO_H
 
-#include "os.h"
+#include "common/os.h"
 
-#include "common.h"
-#include "pr_generic.h"
-#include "smart_pointers.h"
+#include "common/common.h"
+#include "merge/pr_generic.h"
+#include "common/smart_pointers.h"
 
 class ra_packetizer_c: public generic_packetizer_c {
 private:
@@ -37,7 +37,7 @@ public:
   virtual const char *get_format_name() {
     return "RealAudio";
   }
-  virtual connection_result_e can_connect_to(generic_packetizer_c *src, string &error_message);
+  virtual connection_result_e can_connect_to(generic_packetizer_c *src, std::string &error_message);
 };
 
 #endif // __P_REALAUDIO_H
