@@ -14,20 +14,21 @@
 #ifndef __P_TEXTSUBS_H
 #define __P_TEXTSUBS_H
 
-#include "os.h"
+#include "common/os.h"
 
 #include <boost/regex.hpp>
 #include <string>
 
-#include "common.h"
-#include "pr_generic.h"
-#include "smart_pointers.h"
+#include "common/common.h"
+#include "merge/pr_generic.h"
+#include "common/smart_pointers.h"
 
 class textsubs_packetizer_c: public generic_packetizer_c {
 private:
-  int m_packetno, m_cc_utf8;
+  int m_packetno;
+  charset_converter_cptr m_cc_utf8;
   memory_cptr m_global_data;
-  string m_codec_id;
+  std::string m_codec_id;
   bool m_recode;
 
 public:
@@ -41,7 +42,7 @@ public:
   virtual const char *get_format_name() {
     return "text subtitle";
   }
-  virtual connection_result_e can_connect_to(generic_packetizer_c *src, string &error_message);
+  virtual connection_result_e can_connect_to(generic_packetizer_c *src, std::string &error_message);
 
 private:
   static boost::regex s_re_remove_cr, s_re_translate_nl, s_re_remove_trailing_nl;
